@@ -3,7 +3,7 @@
  * extract.c - extracting archives
  *
  * Copyright © 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -120,9 +120,9 @@ extracthalf(const char *debar, const char *dir,
   int arfd;
   struct stat stab;
   char nlc;
-  int adminmember;
+  int adminmember = -1;
   bool header_done;
-  enum compressor_type decompressor = compressor_type_gzip;
+  enum compressor_type decompressor = COMPRESSOR_TYPE_GZIP;
 
   arfd = open(debar, O_RDONLY);
   if (arfd < 0)
@@ -178,21 +178,37 @@ extracthalf(const char *debar, const char *dir,
         if (fd_skip(arfd, memberlen + (memberlen & 1), &err) < 0)
           ohshit(_("cannot skip archive member from '%s': %s"), debar, err.str);
       } else {
-	if (strncmp(arh.ar_name, ADMINMEMBER, sizeof(arh.ar_name)) == 0)
+        if (strncmp(arh.ar_name, ADMINMEMBER, strlen(ADMINMEMBER)) == 0) {
+          const char *extension = arh.ar_name + strlen(ADMINMEMBER);
+
 	  adminmember = 1;
-	else {
-	  adminmember = -1;
+          decompressor = compressor_find_by_extension(extension);
+          if (decompressor != COMPRESSOR_TYPE_NONE &&
+              decompressor != COMPRESSOR_TYPE_GZIP &&
+              decompressor != COMPRESSOR_TYPE_XZ)
+            ohshit(_("archive '%s' uses unknown compression for member '%.*s', "
+                     "giving up"),
+                   debar, (int)sizeof(arh.ar_name), arh.ar_name);
+        } else {
+          if (adminmember != 1)
+            ohshit(_("archive '%s' has premature member '%.*s' before '%s', "
+                     "giving up"),
+                   debar, (int)sizeof(arh.ar_name), arh.ar_name, ADMINMEMBER);
 
 	  if (strncmp(arh.ar_name, DATAMEMBER, strlen(DATAMEMBER)) == 0) {
 	    const char *extension = arh.ar_name + strlen(DATAMEMBER);
 
 	    adminmember= 0;
 	    decompressor = compressor_find_by_extension(extension);
-	  }
-
-          if (adminmember == -1 || decompressor == compressor_type_unknown)
-            ohshit(_("archive '%.250s' contains not understood data member %.*s, giving up"),
-                   debar, (int)sizeof(arh.ar_name), arh.ar_name);
+            if (decompressor == COMPRESSOR_TYPE_UNKNOWN)
+              ohshit(_("archive '%s' uses unknown compression for member '%.*s', "
+                       "giving up"),
+                     debar, (int)sizeof(arh.ar_name), arh.ar_name);
+          } else {
+            ohshit(_("archive '%s' has premature member '%.*s' before '%s', "
+                     "giving up"),
+                   debar, (int)sizeof(arh.ar_name), arh.ar_name, DATAMEMBER);
+          }
         }
         if (adminmember == 1) {
           if (ctrllennum != 0)
@@ -360,29 +376,6 @@ extracthalf(const char *debar, const char *dir,
   }
 }
 
-static int
-controlextractvextract(int admin, enum dpkg_tar_options taroptions,
-                       const char *const *argv)
-{
-  const char *debar, *dir;
-
-  debar = *argv++;
-  if (debar == NULL)
-    badusage(_("--%s needs a .deb filename argument"),cipaction->olong);
-  dir = *argv++;
-  if (!dir) {
-    if (admin)
-      dir = EXTRACTCONTROLDIR;
-    else ohshit(_("--%s needs a target directory.\n"
-                "Perhaps you should be using dpkg --install ?"),cipaction->olong);
-  } else if (*argv) {
-    badusage(_("--%s takes at most two arguments (.deb and directory)"),cipaction->olong);
-  }
-  extracthalf(debar, dir, taroptions, admin);
-
-  return 0;
-}
-
 int
 do_fsystarfile(const char *const *argv)
 {
@@ -401,18 +394,50 @@ do_fsystarfile(const char *const *argv)
 int
 do_control(const char *const *argv)
 {
-  return controlextractvextract(1, DPKG_TAR_EXTRACT, argv);
+  const char *debar, *dir;
+
+  debar = *argv++;
+  if (debar == NULL)
+    badusage(_("--%s needs a .deb filename argument"), cipaction->olong);
+
+  dir = *argv++;
+  if (dir == NULL)
+    dir = EXTRACTCONTROLDIR;
+  else if (*argv)
+    badusage(_("--%s takes at most two arguments (.deb and directory)"),
+             cipaction->olong);
+
+  extracthalf(debar, dir, DPKG_TAR_EXTRACT, 1);
+
+  return 0;
 }
 
 int
 do_extract(const char *const *argv)
 {
+  const char *debar, *dir;
   enum dpkg_tar_options options = DPKG_TAR_EXTRACT | DPKG_TAR_PERMS;
 
   if (opt_verbose)
     options |= DPKG_TAR_LIST;
 
-  return controlextractvextract(0, options, argv);
+  debar = *argv++;
+  if (debar == NULL)
+    badusage(_("--%s needs .deb filename and directory arguments"),
+             cipaction->olong);
+
+  dir = *argv++;
+  if (dir == NULL)
+    badusage(_("--%s needs a target directory.\n"
+               "Perhaps you should be using dpkg --install ?"),
+             cipaction->olong);
+  else if (*argv)
+    badusage(_("--%s takes at most two arguments (.deb and directory)"),
+             cipaction->olong);
+
+  extracthalf(debar, dir, options, 0);
+
+  return 0;
 }
 
 int
@@ -432,7 +457,8 @@ do_raw_extract(const char *const *argv)
 
   debar = *argv++;
   if (debar == NULL)
-    badusage(_("--%s needs a .deb filename argument"), cipaction->olong);
+    badusage(_("--%s needs .deb filename and directory arguments"),
+             cipaction->olong);
 
   dir = *argv++;
   if (dir == NULL)
