@@ -17,19 +17,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <compat.h>
 
 #include <sys/ioctl.h>
-#include <sys/termios.h>
 
 #include <assert.h>
 #include <errno.h>
 #include <ctype.h>
 #include <string.h>
+#include <termios.h>
 #include <unistd.h>
 #include <stdio.h>
 
@@ -49,7 +49,7 @@ void mywerase(WINDOW *win) {
   wmove(win,0,0);
 }
 
-baselist *baselist::signallist= 0;
+baselist *baselist::signallist = nullptr;
 void baselist::sigwinchhandler(int) {
   int save_errno = errno;
   struct winsize size;
@@ -68,9 +68,11 @@ static void cu_sigwinch(int, void **argv) {
   struct sigaction *osigactp= (struct sigaction*)argv[0];
   sigset_t *oblockedp= (sigset_t*)argv[1];
 
-  if (sigaction(SIGWINCH,osigactp,0)) ohshite(_("failed to restore old SIGWINCH sigact"));
+  if (sigaction(SIGWINCH, osigactp, nullptr))
+    ohshite(_("failed to restore old SIGWINCH sigact"));
   delete osigactp;
-  if (sigprocmask(SIG_SETMASK,oblockedp,0)) ohshite(_("failed to restore old signal mask"));
+  if (sigprocmask(SIG_SETMASK, oblockedp, nullptr))
+    ohshite(_("failed to restore old signal mask"));
   delete oblockedp;
 }
 
@@ -80,17 +82,21 @@ void baselist::setupsigwinch() {
 
   osigactp= new(struct sigaction);
   oblockedp= new(sigset_t);
-  if (sigprocmask(0,0,oblockedp)) ohshite(_("failed to get old signal mask"));
-  if (sigaction(SIGWINCH,0,osigactp)) ohshite(_("failed to get old SIGWINCH sigact"));
+  if (sigprocmask(0, nullptr, oblockedp))
+    ohshite(_("failed to get old signal mask"));
+  if (sigaction(SIGWINCH, nullptr, osigactp))
+    ohshite(_("failed to get old SIGWINCH sigact"));
 
-  push_cleanup(cu_sigwinch,~0, 0,0, 2,(void*)osigactp,(void*)oblockedp);
+  push_cleanup(cu_sigwinch, ~0, nullptr, 0, 2, osigactp, oblockedp);
 
-  if (sigprocmask(SIG_BLOCK,&sigwinchset,0)) ohshite(_("failed to block SIGWINCH"));
+  if (sigprocmask(SIG_BLOCK, &sigwinchset, nullptr))
+    ohshite(_("failed to block SIGWINCH"));
   memset(&nsigact,0,sizeof(nsigact));
   nsigact.sa_handler= sigwinchhandler;
   sigemptyset(&nsigact.sa_mask);
 //nsigact.sa_flags= SA_INTERRUPT;
-  if (sigaction(SIGWINCH,&nsigact,0)) ohshite(_("failed to set new SIGWINCH sigact"));
+  if (sigaction(SIGWINCH, &nsigact, nullptr))
+    ohshite(_("failed to set new SIGWINCH sigact"));
 }
 
 void baselist::setheights() {
@@ -211,7 +217,7 @@ void baselist::enddisplay() {
   delwin(thisstatepad);
   delwin(infopad);
   wmove(stdscr,ymax,0); wclrtoeol(stdscr);
-  listpad= 0;
+  listpad = nullptr;
 }
 
 void baselist::redrawall() {
@@ -241,7 +247,8 @@ baselist::baselist(keybindings *kb) {
   xmax= -1;
   list_height=0; info_height=0;
   topofscreen= 0; leftofscreen= 0;
-  listpad= 0; cursorline= -1;
+  listpad = nullptr;
+  cursorline = -1;
   showinfo= 1;
 
   searchstring[0]= 0;
@@ -253,7 +260,7 @@ void baselist::itd_keys() {
   const int givek= xmax/3;
   bindings->describestart();
   const char **ta;
-  while ((ta= bindings->describenext()) != 0) {
+  while ((ta = bindings->describenext()) != nullptr) {
     const char **tap= ta+1;
     for (;;) {
       waddstr(infopad, gettext(*tap));
@@ -265,6 +272,7 @@ void baselist::itd_keys() {
     if (x >= givek) y++;
     mvwaddstr(infopad, y,givek, ta[0]);
     waddch(infopad,'\n');
+    delete [] ta;
   }
 }
 
@@ -313,7 +321,8 @@ void baselist::refreshinfo() {
 void baselist::wordwrapinfo(int offset, const char *m) {
   int usemax= xmax-5;
   debug(dbg_general, "baselist[%p]::wordwrapinfo(%d, '%s')", this, offset, m);
-  int wrapping=0;
+  bool wrapping = false;
+
   for (;;) {
     int offleft=offset; while (*m == ' ' && offleft>0) { m++; offleft--; }
     const char *p= strchr(m,'\n');
@@ -321,12 +330,13 @@ void baselist::wordwrapinfo(int offset, const char *m) {
     while (l && isspace(m[l-1])) l--;
     if (!l || (*m == '.' && l == 1)) {
       if (wrapping) waddch(infopad,'\n');
-      waddch(infopad,'\n');
-      wrapping= 0;
+      waddch(infopad, '\n');
+      wrapping = false;
     } else if (*m == ' ' || usemax < 10) {
       if (wrapping) waddch(infopad,'\n');
       waddnstr(infopad, m, l);
-      waddch(infopad,'\n'); wrapping= 0;
+      waddch(infopad, '\n');
+      wrapping = false;
     } else {
       int x, y DPKG_ATTR_UNUSED;
 
@@ -354,7 +364,7 @@ void baselist::wordwrapinfo(int offset, const char *m) {
         if (l <= 0) break;
         waddch(infopad,'\n');
       }
-      wrapping= 1;
+      wrapping = true;
     }
     if (!p) break;
     if (getcury(infopad) == (MAX_DISPLAY_INFO - 1)) {
