@@ -17,14 +17,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <compat.h>
 
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/wait.h>
 
 #include <assert.h>
@@ -63,7 +62,7 @@
 
 static const char printforhelp[] = N_("Type dselect --help for help.");
 
-int expertmode= 0;
+bool expertmode = false;
 
 static const char *admindir = ADMINDIR;
 
@@ -83,7 +82,7 @@ static const struct table_t colourtable[]= {
   {"magenta",	COLOR_MAGENTA	},
   {"cyan",	COLOR_CYAN	},
   {"white",	COLOR_WHITE	},
-  {NULL, 0},
+  {nullptr, 0},
 };
 
 static const struct table_t attrtable[]= {
@@ -95,7 +94,7 @@ static const struct table_t attrtable[]= {
   {"bright",	A_BLINK		}, // on some terminals
   {"dim",	A_DIM		},
   {"bold",	A_BOLD		},
-  {NULL, 0},
+  {nullptr, 0},
 };
 
 /* A slightly confusing mapping from dselect's internal names to
@@ -113,7 +112,7 @@ static const struct table_t screenparttable[]= {
   {"infodesc",		info_head	},
   {"infofoot",		whatinfo	},
   {"helpscreen",	helpscreen	},
-  {NULL, 0},
+  {nullptr, 0},
 };
 
 /* Historical (patriotic?) colours. */
@@ -150,8 +149,8 @@ static const menuentry menuentries[]= {
   { "config",	N_("c"),	N_("[C]onfig"),	N_("Configure any packages that are unconfigured."),	&urq_config  },
   { "remove",	N_("r"),	N_("[R]emove"),	N_("Remove unwanted software."),			&urq_remove  },
   { "quit",	N_("q"),	N_("[Q]uit"),	N_("Quit dselect."),					&urq_quit    },
-  { 0,		0,  		N_("menu"),	0,							&urq_menu    },
-  { 0 }
+  { nullptr,	nullptr,	N_("menu"),	nullptr,						&urq_menu    },
+  { nullptr }
 };
 
 static const char programdesc[]=
@@ -223,20 +222,24 @@ usage(const struct cmdinfo *ci, const char *value)
 /* These are called by C code, so need to have C calling convention */
 extern "C" {
 
-  static void setdebug(const struct cmdinfo*, const char *v) {
+  static void
+  set_debug(const struct cmdinfo*, const char *v)
+  {
     FILE *fp;
 
     fp = fopen(v, "a");
     if (!fp)
       ohshite(_("couldn't open debug file `%.255s'\n"), v);
-    setvbuf(fp, 0, _IONBF, 0);
+    dpkg_set_report_buffer(fp);
 
     debug_set_output(fp);
     debug_set_mask(dbg_general | dbg_depcon);
   }
 
-  static void setexpert(const struct cmdinfo*, const char *v) {
-    expertmode= 1;
+  static void
+  set_expert(const struct cmdinfo*, const char *v)
+  {
+    expertmode = true;
   }
 
   static int
@@ -244,7 +247,7 @@ extern "C" {
   {
     int i;
 
-    for (i= 0;  item && (table[i].name!=NULL); i++)
+    for (i = 0; item && (table[i].name != nullptr); i++)
       if (strcasecmp(item, table[i].name) == 0)
         return table[i].num;
 
@@ -259,39 +262,41 @@ extern "C" {
    *            --color colheads:,green:bright
    *            --color selstate::reverse  // doesn't work FIXME
    */
-  static void setcolor(const struct cmdinfo*, const char *string) {
+  static void
+  set_color(const struct cmdinfo*, const char *string)
+  {
     char *s;
     char *colours, *attributes, *attrib, *colourname;
     int screenpart, aval;
 
     s = m_strdup(string); // strtok modifies strings, keep string const
     screenpart= findintable(screenparttable, strtok(s, ":"), _("screen part"));
-    colours= strtok(NULL, ":");
-    attributes= strtok(NULL, ":");
+    colours = strtok(nullptr, ":");
+    attributes = strtok(nullptr, ":");
 
-    if ((colours == NULL || ! strlen(colours)) &&
-        (attributes == NULL || ! strlen(attributes))) {
+    if ((colours == nullptr || ! strlen(colours)) &&
+        (attributes == nullptr || ! strlen(attributes))) {
        ohshit(_("null colour specification"));
     }
 
-    if (colours != NULL && strlen(colours)) {
+    if (colours != nullptr && strlen(colours)) {
       colourname= strtok(colours, ",");
-      if (colourname != NULL && strlen(colourname)) {
+      if (colourname != nullptr && strlen(colourname)) {
         // normalize attributes to prevent confusion
         color[screenpart].attr= A_NORMAL;
        color[screenpart].fore=findintable(colourtable, colourname, _("colour"));
       }
-      colourname= strtok(NULL, ",");
-      if (colourname != NULL && strlen(colourname)) {
+      colourname = strtok(nullptr, ",");
+      if (colourname != nullptr && strlen(colourname)) {
         color[screenpart].attr= A_NORMAL;
         color[screenpart].back=findintable(colourtable, colourname, _("colour"));
       }
     }
 
-    if (attributes != NULL && strlen(attributes)) {
+    if (attributes != nullptr && strlen(attributes)) {
       for (attrib= strtok(attributes, "+");
-           attrib != NULL && strlen(attrib);
-          attrib= strtok(NULL, "+")) {
+           attrib != nullptr && strlen(attrib);
+           attrib = strtok(nullptr, "+")) {
                aval=findintable(attrtable, attrib, _("colour attribute"));
                if (aval == A_NORMAL) // set to normal
                        color[screenpart].attr= aval;
@@ -299,22 +304,24 @@ extern "C" {
                        color[screenpart].attr= color[screenpart].attr | aval;
       }
     }
+
+    free(s);
   }
 
 } /* End of extern "C" */
 
 static const struct cmdinfo cmdinfos[]= {
-  { "admindir",     0,   1,  0,  &admindir,  0               },
-  { "debug",       'D',  1,  0,  0,          setdebug        },
-  { "expert",      'E',  0,  0,  0,          setexpert       },
-  { "help",        '?',  0,  0,  0,          usage           },
-  { "version",      0,   0,  0,  0,          printversion    },
-  { "color",        0,   1,  0,  0,          setcolor        }, /* US spelling */
-  { "colour",       0,   1,  0,  0,          setcolor        }, /* UK spelling */
-  { 0,              0,   0,  0,  0,          0               }
+  { "admindir",     0,  1, nullptr,  &admindir, nullptr      },
+  { "debug",       'D', 1, nullptr,  nullptr,   set_debug    },
+  { "expert",      'E', 0, nullptr,  nullptr,   set_expert   },
+  { "help",        '?', 0, nullptr,  nullptr,   usage        },
+  { "version",      0,  0, nullptr,  nullptr,   printversion },
+  { "color",        0,  1, nullptr,  nullptr,   set_color    }, /* US spelling */
+  { "colour",       0,  1, nullptr,  nullptr,   set_color    }, /* UK spelling */
+  { nullptr,        0,  0, nullptr,  nullptr,   nullptr      }
 };
 
-static int cursesareon= 0;
+static bool cursesareon = false;
 void curseson() {
   if (!cursesareon) {
     const char *cup, *smso;
@@ -334,7 +341,7 @@ void curseson() {
       ohshit(_("terminal lacks necessary features, giving up"));
     }
   }
-  cursesareon= 1;
+  cursesareon = true;
 }
 
 void cursesoff() {
@@ -343,7 +350,7 @@ void cursesoff() {
     refresh();
     endwin();
   }
-  cursesareon=0;
+  cursesareon = false;
 }
 
 extern void *operator new(size_t size) {
@@ -509,22 +516,19 @@ dselect_catch_fatal_error()
 int
 main(int, const char *const *argv)
 {
-  setlocale(LC_ALL, "");
-  bindtextdomain(DSELECT, LOCALEDIR);
-  textdomain(DSELECT);
-
+  dpkg_locales_init(DSELECT);
   dpkg_set_progname(DSELECT);
 
-  push_error_context_func(dselect_catch_fatal_error, print_fatal_error, 0);
+  push_error_context_func(dselect_catch_fatal_error, print_fatal_error, nullptr);
 
-  loadcfgfile(DSELECT, cmdinfos);
-  myopt(&argv, cmdinfos, printforhelp);
+  dpkg_options_load(DSELECT, cmdinfos);
+  dpkg_options_parse(&argv, cmdinfos, printforhelp);
 
   admindir = dpkg_db_set_dir(admindir);
 
   if (*argv) {
     const char *a;
-    while ((a= *argv++) != 0) {
+    while ((a = *argv++) != nullptr) {
       const menuentry *me = menuentries;
       while (me->command && strcmp(me->command, a))
         me++;
@@ -536,6 +540,7 @@ main(int, const char *const *argv)
   }
 
   cursesoff();
-  standard_shutdown();
+  dpkg_program_done();
+
   return(0);
 }
