@@ -4,7 +4,7 @@
  *
  * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  * Copyright © 2001 Wichert Akkerman
- * Copyright © 2006,2008-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006,2008-2014 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -19,7 +19,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /* FIXME: Don't write uninteresting packages. */
@@ -46,6 +46,13 @@
 #include <dpkg/dir.h>
 #include <dpkg/parsedump.h>
 
+static inline void
+varbuf_add_fieldname(struct varbuf *vb, const struct fieldinfo *fip)
+{
+  varbuf_add_str(vb, fip->name);
+  varbuf_add_str(vb, ": ");
+}
+
 void
 w_name(struct varbuf *vb,
        const struct pkginfo *pkg, const struct pkgbin *pkgbin,
@@ -82,9 +89,9 @@ w_configversion(struct varbuf *vb,
     return;
   if (!dpkg_version_is_informative(&pkg->configversion))
     return;
-  if (pkg->status == stat_installed ||
-      pkg->status == stat_notinstalled ||
-      pkg->status == stat_triggerspending)
+  if (pkg->status == PKG_STAT_INSTALLED ||
+      pkg->status == PKG_STAT_NOTINSTALLED ||
+      pkg->status == PKG_STAT_TRIGGERSPENDING)
     return;
   if (flags&fw_printheader)
     varbuf_add_str(vb, "Config-Version: ");
@@ -125,10 +132,8 @@ w_charfield(struct varbuf *vb,
 
   if (str_is_unset(value))
     return;
-  if (flags&fw_printheader) {
-    varbuf_add_str(vb, fip->name);
-    varbuf_add_str(vb, ": ");
-  }
+  if (flags & fw_printheader)
+    varbuf_add_fieldname(vb, fip);
   varbuf_add_str(vb, value);
   if (flags&fw_printheader)
     varbuf_add_char(vb, '\n');
@@ -172,10 +177,8 @@ w_booleandefno(struct varbuf *vb,
   if ((flags & fw_printheader) && !value)
     return;
 
-  if (flags & fw_printheader) {
-    varbuf_add_str(vb, fip->name);
-    varbuf_add_str(vb, ": ");
-  }
+  if (flags & fw_printheader)
+    varbuf_add_fieldname(vb, fip);
 
   varbuf_add_str(vb, value ? "yes" : "no");
 
@@ -193,10 +196,8 @@ w_multiarch(struct varbuf *vb,
   if ((flags & fw_printheader) && !value)
     return;
 
-  if (flags & fw_printheader) {
-    varbuf_add_str(vb, fip->name);
-    varbuf_add_str(vb, ": ");
-  }
+  if (flags & fw_printheader)
+    varbuf_add_fieldname(vb, fip);
 
   varbuf_add_str(vb, multiarchinfos[value].name);
 
@@ -211,15 +212,13 @@ w_architecture(struct varbuf *vb,
 {
   if (!pkgbin->arch)
     return;
-  if (pkgbin->arch->type == arch_none)
+  if (pkgbin->arch->type == DPKG_ARCH_NONE)
     return;
-  if (pkgbin->arch->type == arch_empty)
+  if (pkgbin->arch->type == DPKG_ARCH_EMPTY)
     return;
 
-  if (flags & fw_printheader) {
-    varbuf_add_str(vb, fip->name);
-    varbuf_add_str(vb, ": ");
-  }
+  if (flags & fw_printheader)
+    varbuf_add_fieldname(vb, fip);
   varbuf_add_str(vb, pkgbin->arch->name);
   if (flags & fw_printheader)
     varbuf_add_char(vb, '\n');
@@ -230,14 +229,12 @@ w_priority(struct varbuf *vb,
            const struct pkginfo *pkg, const struct pkgbin *pkgbin,
            enum fwriteflags flags, const struct fieldinfo *fip)
 {
-  if (pkg->priority == pri_unknown)
+  if (pkg->priority == PKG_PRIO_UNKNOWN)
     return;
-  assert(pkg->priority <= pri_unknown);
+  assert(pkg->priority <= PKG_PRIO_UNKNOWN);
   if (flags&fw_printheader)
     varbuf_add_str(vb, "Priority: ");
-  varbuf_add_str(vb, pkg->priority == pri_other ?
-                     pkg->otherpriority :
-                     priorityinfos[pkg->priority].name);
+  varbuf_add_str(vb, pkg_priority_name(pkg));
   if (flags&fw_printheader)
     varbuf_add_char(vb, '\n');
 }
@@ -249,30 +246,30 @@ w_status(struct varbuf *vb,
 {
   if (pkgbin != &pkg->installed)
     return;
-  assert(pkg->want <= want_purge);
-  assert(pkg->eflag <= eflag_reinstreq);
+  assert(pkg->want <= PKG_WANT_PURGE);
+  assert(pkg->eflag <= PKG_EFLAG_REINSTREQ);
 
 #define PEND pkg->trigpend_head
 #define AW pkg->trigaw.head
   switch (pkg->status) {
-  case stat_notinstalled:
-  case stat_configfiles:
+  case PKG_STAT_NOTINSTALLED:
+  case PKG_STAT_CONFIGFILES:
     assert(!PEND);
     assert(!AW);
     break;
-  case stat_halfinstalled:
-  case stat_unpacked:
-  case stat_halfconfigured:
+  case PKG_STAT_HALFINSTALLED:
+  case PKG_STAT_UNPACKED:
+  case PKG_STAT_HALFCONFIGURED:
     assert(!PEND);
     break;
-  case stat_triggersawaited:
+  case PKG_STAT_TRIGGERSAWAITED:
     assert(AW);
     break;
-  case stat_triggerspending:
+  case PKG_STAT_TRIGGERSPENDING:
     assert(PEND);
     assert(!AW);
     break;
-  case stat_installed:
+  case PKG_STAT_INSTALLED:
     assert(!PEND);
     assert(!AW);
     break;
@@ -284,11 +281,11 @@ w_status(struct varbuf *vb,
 
   if (flags&fw_printheader)
     varbuf_add_str(vb, "Status: ");
-  varbuf_add_str(vb, wantinfos[pkg->want].name);
+  varbuf_add_str(vb, pkg_want_name(pkg));
   varbuf_add_char(vb, ' ');
-  varbuf_add_str(vb, eflaginfos[pkg->eflag].name);
+  varbuf_add_str(vb, pkg_eflag_name(pkg));
   varbuf_add_char(vb, ' ');
-  varbuf_add_str(vb, statusinfos[pkg->status].name);
+  varbuf_add_str(vb, pkg_status_name(pkg));
   if (flags&fw_printheader)
     varbuf_add_char(vb, '\n');
 }
@@ -305,22 +302,22 @@ void varbufdependency(struct varbuf *vb, struct dependency *dep) {
     varbuf_add_str(vb, dop->ed->name);
     if (!dop->arch_is_implicit)
       varbuf_add_archqual(vb, dop->arch);
-    if (dop->verrel != dpkg_relation_none) {
+    if (dop->verrel != DPKG_RELATION_NONE) {
       varbuf_add_str(vb, " (");
       switch (dop->verrel) {
-      case dpkg_relation_eq:
+      case DPKG_RELATION_EQ:
         varbuf_add_char(vb, '=');
         break;
-      case dpkg_relation_ge:
+      case DPKG_RELATION_GE:
         varbuf_add_str(vb, ">=");
         break;
-      case dpkg_relation_le:
+      case DPKG_RELATION_LE:
         varbuf_add_str(vb, "<=");
         break;
-      case dpkg_relation_gt:
+      case DPKG_RELATION_GT:
         varbuf_add_str(vb, ">>");
         break;
-      case dpkg_relation_lt:
+      case DPKG_RELATION_LT:
         varbuf_add_str(vb, "<<");
         break;
       default:
@@ -338,24 +335,23 @@ w_dependency(struct varbuf *vb,
              const struct pkginfo *pkg, const struct pkgbin *pkgbin,
              enum fwriteflags flags, const struct fieldinfo *fip)
 {
-  char fnbuf[50];
-  const char *depdel;
   struct dependency *dyp;
+  bool dep_found = false;
 
-  if (flags&fw_printheader)
-    sprintf(fnbuf,"%s: ",fip->name);
-  else
-    fnbuf[0] = '\0';
-
-  depdel= fnbuf;
   for (dyp = pkgbin->depends; dyp; dyp = dyp->next) {
     if (dyp->type != fip->integer) continue;
     assert(dyp->up == pkg);
-    varbuf_add_str(vb, depdel);
-    depdel = ", ";
+
+    if (dep_found) {
+      varbuf_add_str(vb, ", ");
+    } else {
+      if (flags & fw_printheader)
+        varbuf_add_fieldname(vb, fip);
+      dep_found = true;
+    }
     varbufdependency(vb,dyp);
   }
-  if ((flags&fw_printheader) && (depdel!=fnbuf))
+  if ((flags & fw_printheader) && dep_found)
     varbuf_add_char(vb, '\n');
 }
 
@@ -394,8 +390,8 @@ w_trigpend(struct varbuf *vb,
   if (pkgbin == &pkg->available || !pkg->trigpend_head)
     return;
 
-  assert(pkg->status >= stat_triggersawaited &&
-         pkg->status <= stat_triggerspending);
+  assert(pkg->status >= PKG_STAT_TRIGGERSAWAITED &&
+         pkg->status <= PKG_STAT_TRIGGERSPENDING);
 
   if (flags & fw_printheader)
     varbuf_add_str(vb, "Triggers-Pending:");
@@ -417,8 +413,8 @@ w_trigaw(struct varbuf *vb,
   if (pkgbin == &pkg->available || !pkg->trigaw.head)
     return;
 
-  assert(pkg->status > stat_configfiles &&
-         pkg->status <= stat_triggersawaited);
+  assert(pkg->status > PKG_STAT_CONFIGFILES &&
+         pkg->status <= PKG_STAT_TRIGGERSAWAITED);
 
   if (flags & fw_printheader)
     varbuf_add_str(vb, "Triggers-Awaited:");
@@ -484,7 +480,7 @@ writedb(const char *filename, enum writedb_flags flags)
 
   which = (flags & wdb_dump_available) ? "available" : "status";
 
-  file = atomic_file_new(filename, aff_backup);
+  file = atomic_file_new(filename, ATOMIC_FILE_BACKUP);
   atomic_file_open(file);
   if (setvbuf(file->fp, writebuf, _IOFBF, sizeof(writebuf)))
     ohshite(_("unable to set buffering on %s database file"), which);
