@@ -17,7 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef LIBDPKG_PARSEDUMP_H
@@ -52,6 +52,7 @@ struct parsedb_state {
 	char *dataptr;
 	char *endptr;
 	const char *filename;
+	int fd;
 	int lno;
 };
 
@@ -68,9 +69,16 @@ struct field_state {
 	int *fieldencountered;
 };
 
-void parse_open(struct parsedb_state *ps, const char *filename,
-                enum parsedbflags flags);
-void parse_close(struct parsedb_state *ps);
+struct parsedb_state *
+parsedb_new(const char *filename, int fd, enum parsedbflags flags);
+struct parsedb_state *
+parsedb_open(const char *filename, enum parsedbflags flags);
+void
+parsedb_load(struct parsedb_state *ps);
+int
+parsedb_parse(struct parsedb_state *ps, struct pkginfo **pkgp);
+void
+parsedb_close(struct parsedb_state *ps);
 
 typedef void parse_field_func(struct parsedb_state *ps, struct field_state *fs,
                               void *parse_obj);
