@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 =encoding utf8
 
@@ -107,10 +107,10 @@ sub reset_parse_errors {
     $self->{parse_errors} = [];
 }
 
-=item $c->parse_error($line_nr, $error, [$line])
+=item $c->parse_error($file, $line_nr, $error, [$line])
 
-Record a new parse error at line $line_nr. The error message is specified
-with $error and a copy of the line can be recorded in $line.
+Record a new parse error in $file at line $line_nr. The error message is
+specified with $error and a copy of the line can be recorded in $line.
 
 =cut
 
@@ -168,7 +168,7 @@ sub get_parse_errors {
 	my $res = '';
 	foreach my $e (@{$self->{parse_errors}}) {
 	    if ($e->[3]) {
-		$res .= report(_g('warning'),_g("%s(l%s): %s\nLINE: %s"), @$e );
+		$res .= report(_g('warning'), _g("%s(l%s): %s\nLINE: %s"), @$e);
 	    } else {
 		$res .= report(_g('warning'), _g('%s(l%s): %s'), @$e);
 	    }
@@ -462,12 +462,12 @@ sub output {
     my $str = '';
     foreach my $entry (@{$self}) {
 	my $text = $entry->output();
-	print $fh $text if defined $fh;
+	print { $fh } $text if defined $fh;
 	$str .= $text if defined wantarray;
     }
     my $text = $self->get_unparsed_tail();
     if (defined $text) {
-	print $fh $text if defined $fh;
+	print { $fh } $text if defined $fh;
 	$str .= $text if defined wantarray;
     }
     return $str;
