@@ -1,6 +1,7 @@
 # Copyright © 1996 Ian Jackson
 # Copyright © 2005 Frank Lichtenheld <frank@lichtenheld.de>
 # Copyright © 2009 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2012-2013 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 =encoding utf8
 
@@ -49,7 +50,7 @@ use Dpkg::Gettext;
 use Dpkg::File;
 use Dpkg::Changelog qw(:util);
 use parent qw(Dpkg::Changelog);
-use Dpkg::Changelog::Entry::Debian qw($regex_header $regex_trailer);
+use Dpkg::Changelog::Entry::Debian qw(match_header match_trailer);
 
 use constant {
     FIRST_HEADING => _g('first heading'),
@@ -58,9 +59,9 @@ use constant {
     CHANGES_OR_TRAILER => _g('more change data or trailer'),
 };
 
-=pod
+=over 4
 
-=head3 $c->parse($fh, $description)
+=item $c->parse($fh, $description)
 
 Read the filehandle and parse a Debian changelog in it. Returns the number
 of changelog entries that have been parsed with success.
@@ -80,11 +81,11 @@ sub parse {
     my $entry = Dpkg::Changelog::Entry::Debian->new();
     my @blanklines = ();
     my $unknowncounter = 1; # to make version unique, e.g. for using as id
+    local $_;
 
     while (<$fh>) {
 	chomp;
-	if ($_ =~ $regex_header) {
-	    (my $options = $4) =~ s/^\s+//;
+	if (match_header($_)) {
 	    unless ($expect eq FIRST_HEADING || $expect eq NEXT_OR_EOF) {
 		$self->parse_error($file, $.,
 		    sprintf(_g('found start of entry where expected %s'),
@@ -101,7 +102,7 @@ sub parse {
 	    }
 	    $expect= START_CHANGES;
 	    @blanklines = ();
-	} elsif (m/^(;;\s*)?Local variables:/io) {
+	} elsif (m/^(?:;;\s*)?Local variables:/io) {
 	    last; # skip Emacs variables at end of file
 	} elsif (m/^vim:/io) {
 	    last; # skip vim variables at end of file
@@ -111,11 +112,11 @@ sub parse {
 	    next; # skip comments, even that's not supported
 	} elsif (m{^/\*.*\*/}o) {
 	    next; # more comments
-	} elsif (m/^(\w+\s+\w+\s+\d{1,2} \d{1,2}:\d{1,2}:\d{1,2}\s+[\w\s]*\d{4})\s+(.*)\s+(<|\()(.*)(\)|>)/o
-		 || m/^(\w+\s+\w+\s+\d{1,2},?\s*\d{4})\s+(.*)\s+(<|\()(.*)(\)|>)/o
-		 || m/^(\w[-+0-9a-z.]*) \(([^\(\) \t]+)\)\;?/io
-		 || m/^([\w.+-]+)(-| )(\S+) Debian (\S+)/io
-		 || m/^Changes from version (.*) to (.*):/io
+	} elsif (m/^(?:\w+\s+\w+\s+\d{1,2} \d{1,2}:\d{1,2}:\d{1,2}\s+[\w\s]*\d{4})\s+(?:.*)\s+[<\(](?:.*)[\)>]/o
+		 || m/^(?:\w+\s+\w+\s+\d{1,2},?\s*\d{4})\s+(?:.*)\s+[<\(](?:.*)[\)>]/o
+		 || m/^(?:\w[-+0-9a-z.]*) \((?:[^\(\) \t]+)\)\;?/io
+		 || m/^(?:[\w.+-]+)[- ](?:\S+) Debian (?:\S+)/io
+		 || m/^Changes from version (?:.*) to (?:.*):/io
 		 || m/^Changes for [\w.+-]+-[\w.+-]+:?\s*$/io
 		 || m/^Old Changelog:\s*$/io
 		 || m/^(?:\d+:)?\w[\w.+~-]*:?\s*$/o) {
@@ -125,7 +126,7 @@ sub parse {
 	    $self->set_unparsed_tail("$_\n" . file_slurp($fh));
 	} elsif (m/^\S/) {
 	    $self->parse_error($file, $., _g('badly formatted heading line'), "$_");
-	} elsif ($_ =~ $regex_trailer) {
+	} elsif (match_trailer($_)) {
 	    unless ($expect eq CHANGES_OR_TRAILER) {
 		$self->parse_error($file, $.,
 		    sprintf(_g('found trailer where expected %s'), $expect), "$_");
@@ -139,7 +140,7 @@ sub parse {
 	    $expect = NEXT_OR_EOF;
 	} elsif (m/^ \-\-/) {
 	    $self->parse_error($file, $., _g('badly formatted trailer line'), "$_");
-	} elsif (m/^\s{2,}(\S)/) {
+	} elsif (m/^\s{2,}(?:\S)/) {
 	    unless ($expect eq START_CHANGES or $expect eq CHANGES_OR_TRAILER) {
 		$self->parse_error($file, $., sprintf(_g('found change data' .
 		    ' where expected %s'), $expect), "$_");
@@ -191,6 +192,8 @@ sub parse {
 1;
 __END__
 
+=back
+
 =head1 SEE ALSO
 
 Dpkg::Changelog
