@@ -12,7 +12,7 @@
 #    GNU General Public License for more details.
 #
 #    You should have received a copy of the GNU General Public License
-#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 =encoding utf8
 
@@ -25,7 +25,7 @@ Dpkg::Changelog::Parse - generic changelog parser for dpkg-parsechangelog
 This module provides a single function changelog_parse() which reproduces
 all the features of dpkg-parsechangelog.
 
-=head2 Functions
+=head2 FUNCTIONS
 
 =cut
 
@@ -44,7 +44,9 @@ use Dpkg::Control::Changelog;
 use Exporter qw(import);
 our @EXPORT = qw(changelog_parse);
 
-=head3 my $fields = changelog_parse(%opt)
+=over 4
+
+=item my $fields = changelog_parse(%opt)
 
 This function will parse a changelog. In list context, it return as many
 Dpkg::Control object as the parser did output. In scalar context, it will
@@ -99,8 +101,11 @@ sub changelog_parse {
     my $changelogfile = $options{file};
 
     # Extract the format from the changelog file if possible
-    unless($force or ($changelogfile eq '-')) {
-	open(my $format_fh, '-|', 'tail', '-n', '40', $changelogfile);
+    unless ($force or ($changelogfile eq '-')) {
+	local $_;
+
+	open(my $format_fh, '-|', 'tail', '-n', '40', $changelogfile)
+	    or syserr(_g('cannot create pipe for %s'), 'tail');
 	while (<$format_fh>) {
 	    $format = $1 if m/\schangelog-format:\s+([0-9a-z]+)\W/;
 	}
@@ -138,7 +143,7 @@ sub changelog_parse {
     my $pid = open(my $parser_fh, '-|');
     syserr(_g('cannot fork for %s'), $parser) unless defined $pid;
     if (not $pid) {
-	exec(@exec) || syserr(_g('cannot exec format parser: %s'), $parser);
+	exec(@exec) or syserr(_g('cannot exec format parser: %s'), $parser);
     }
 
     # Get the output into several Dpkg::Control objects
@@ -157,4 +162,8 @@ sub changelog_parse {
     }
 }
 
+=back
+
+=cut
+
 1;
