@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Package;
 
@@ -29,12 +29,15 @@ our @EXPORT = qw(pkg_name_is_illegal);
 sub pkg_name_is_illegal($) {
     my $name = shift || '';
 
-    $name eq '' &&
+    if ($name eq '') {
         return _g('may not be empty string');
-    $name =~ m/[^-+.0-9a-z]/o &&
-        return sprintf(_g("character '%s' not allowed"), $&);
-    $name =~ m/^[0-9a-z]/o ||
+    }
+    if ($name =~ m/[^-+.0-9a-z]/op) {
+        return sprintf(_g("character '%s' not allowed"), ${^MATCH});
+    }
+    if ($name !~ m/^[0-9a-z]/o) {
         return _g('must start with an alphanumeric character');
+    }
 
     return;
 }
