@@ -11,7 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Shlibs::Objdump;
 
@@ -93,6 +93,8 @@ sub has_object {
 	    return $format{$file};
 	} else {
 	    my ($output, %opts, $pid, $res);
+	    local $_;
+
 	    if ($objdump ne 'objdump') {
 		$opts{error_to_file} = '/dev/null';
 	    }
@@ -120,7 +122,7 @@ sub has_object {
 
 sub is_elf {
     my ($file) = @_;
-    open(my $file_fh, '<', $file) || syserr(_g('cannot read %s'), $file);
+    open(my $file_fh, '<', $file) or syserr(_g('cannot read %s'), $file);
     my ($header, $result) = ('', 0);
     if (read($file_fh, $header, 4) == 4) {
 	$result = 1 if ($header =~ /^\177ELF$/);
@@ -179,7 +181,7 @@ sub analyze {
 
     local $ENV{LC_ALL} = 'C';
     open(my $objdump, '-|', $OBJDUMP, '-w', '-f', '-p', '-T', '-R', $file)
-	|| syserr(_g('cannot fork for %s'), $OBJDUMP);
+        or syserr(_g('cannot fork for %s'), $OBJDUMP);
     my $ret = $self->parse_objdump_output($objdump);
     close($objdump);
     return $ret;
@@ -220,7 +222,7 @@ sub parse_objdump_output {
 	    if (/^\S+\s+(\S+)\s+(\S+)\s*$/) {
 		$self->{dynrelocs}{$2} = $1;
 	    } else {
-		warning(_g("Couldn't parse dynamic relocation record: %s"), $_);
+		warning(_g("couldn't parse dynamic relocation record: %s"), $_);
 	    }
 	} elsif ($section eq 'dyninfo') {
 	    if (/^\s*NEEDED\s+(\S+)/) {
@@ -235,10 +237,12 @@ sub parse_objdump_output {
                 # RUNPATH takes precedence over RPATH but is
                 # considered after LD_LIBRARY_PATH while RPATH
                 # is considered before (if RUNPATH is not set).
-                $self->{RPATH} = [ split (/:/, $1) ];
+                my $runpath = $1;
+                $self->{RPATH} = [ split /:/, $runpath ];
 	    } elsif (/^\s*RPATH\s+(\S+)/) {
+                my $rpath = $1;
                 unless (scalar(@{$self->{RPATH}})) {
-                    $self->{RPATH} = [ split (/:/, $1) ];
+                    $self->{RPATH} = [ split /:/, $rpath ];
                 }
 	    }
 	} elsif ($section eq 'none') {
@@ -309,7 +313,7 @@ sub parse_dynamic_symbol {
 
 	my $symbol = {
 		name => $name,
-		version => defined($ver) ? $ver : '',
+		version => $ver // '',
 		section => $sect,
 		dynamic => substr($flags, 5, 1) eq 'D',
 		debug => substr($flags, 5, 1) eq 'd',
@@ -317,7 +321,7 @@ sub parse_dynamic_symbol {
 		weak => substr($flags, 1, 1) eq 'w',
 		local => substr($flags, 0, 1) eq 'l',
 		global => substr($flags, 0, 1) eq 'g',
-		visibility => defined($vis) ? $vis : '',
+		visibility => $vis // '',
 		hidden => '',
 		defined => $sect ne '*UND*'
 	    };
@@ -337,7 +341,7 @@ sub parse_dynamic_symbol {
 	# Ignore some s390-specific output like
 	# REG_G6           g     R *UND*      0000000000000000              #scratch
     } else {
-	warning(_g("Couldn't parse dynamic symbol definition: %s"), $line);
+	warning(_g("couldn't parse dynamic symbol definition: %s"), $line);
     }
 }
 
