@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Shlibs::SymbolFile;
 
@@ -31,33 +31,33 @@ use Dpkg::Arch qw(get_host_arch);
 use parent qw(Dpkg::Interface::Storable);
 
 my %blacklist = (
-    __bss_end__ => 1,			# arm
-    __bss_end => 1,			# arm
-    _bss_end__ => 1,			# arm
-    __bss_start => 1,			# ALL
-    __bss_start__ => 1,			# arm
-    __data_start => 1,			# arm
-    __do_global_ctors_aux => 1,		# ia64
-    __do_global_dtors_aux => 1,		# ia64
-    __do_jv_register_classes => 1,	# ia64
-    _DYNAMIC => 1,			# ALL
-    _edata => 1,			# ALL
-    _end => 1,				# ALL
-    __end__ => 1,			# arm
-    __exidx_end => 1,			# armel
-    __exidx_start => 1,			# armel
-    _fbss => 1,				# mips, mipsel
-    _fdata => 1,			# mips, mipsel
-    _fini => 1,				# ALL
-    _ftext => 1,			# mips, mipsel
-    _GLOBAL_OFFSET_TABLE_ => 1,		# hppa, mips, mipsel
-    __gmon_start__ => 1,		# hppa
-    __gnu_local_gp => 1,		# mips, mipsel
-    _gp => 1,				# mips, mipsel
-    _init => 1,				# ALL
-    _PROCEDURE_LINKAGE_TABLE_ => 1,	# sparc, alpha
-    _SDA2_BASE_ => 1,			# powerpc
-    _SDA_BASE_ => 1,			# powerpc
+    __bss_end__ => 1,                   # arm
+    __bss_end => 1,                     # arm
+    _bss_end__ => 1,                    # arm
+    __bss_start => 1,                   # ALL
+    __bss_start__ => 1,                 # arm
+    __data_start => 1,                  # arm
+    __do_global_ctors_aux => 1,         # ia64
+    __do_global_dtors_aux => 1,         # ia64
+    __do_jv_register_classes => 1,      # ia64
+    _DYNAMIC => 1,                      # ALL
+    _edata => 1,                        # ALL
+    _end => 1,                          # ALL
+    __end__ => 1,                       # arm
+    __exidx_end => 1,                   # armel
+    __exidx_start => 1,                 # armel
+    _fbss => 1,                         # mips, mipsel
+    _fdata => 1,                        # mips, mipsel
+    _fini => 1,                         # ALL
+    _ftext => 1,                        # mips, mipsel
+    _GLOBAL_OFFSET_TABLE_ => 1,         # hppa, mips, mipsel
+    __gmon_start__ => 1,                # hppa
+    __gnu_local_gp => 1,                # mips, mipsel
+    _gp => 1,                           # mips, mipsel
+    _init => 1,                         # ALL
+    _PROCEDURE_LINKAGE_TABLE_ => 1,     # sparc, alpha
+    _SDA2_BASE_ => 1,                   # powerpc
+    _SDA_BASE_ => 1,                    # powerpc
 );
 
 for my $i (14 .. 31) {
@@ -70,14 +70,22 @@ for my $i (14 .. 31) {
     $blacklist{"_savegpr_$i"} = 1;
 }
 
-# Many armel-specific symbols
-$blacklist{"__aeabi_$_"} = 1 foreach (qw(cdcmpeq cdcmple cdrcmple cfcmpeq
-cfcmple cfrcmple d2f d2iz d2lz d2uiz d2ulz dadd dcmpeq dcmpge dcmpgt
-dcmple dcmplt dcmpun ddiv dmul dneg drsub dsub f2d f2iz f2lz f2uiz f2ulz
-fadd fcmpeq fcmpge fcmpgt fcmple fcmplt fcmpun fdiv fmul fneg frsub fsub
-i2d i2f idiv idivmod l2d l2f lasr lcmp ldivmod llsl llsr lmul ui2d ui2f
-uidiv uidivmod ul2d ul2f ulcmp uldivmod unwind_cpp_pr0 unwind_cpp_pr1
-unwind_cpp_pr2 uread4 uread8 uwrite4 uwrite8));
+sub symbol_is_blacklisted {
+    my ($symbol, $include_groups) = @_;
+
+    return 1 if exists $blacklist{$symbol};
+
+    # The ARM Embedded ABI spec states symbols under this namespace as
+    # possibly appearing in output objects.
+    return 1 if not ${$include_groups}{aeabi} and $symbol =~ /^__aeabi_/;
+
+    # The GNU implementation of the OpenMP spec, specifies symbols under
+    # this namespace as possibly appearing in output objects.
+    return 1 if not ${$include_groups}{gomp}
+                and $symbol =~ /^\.gomp_critical_user_/;
+
+    return 0;
+}
 
 sub new {
     my $this = shift;
@@ -223,7 +231,7 @@ sub parse {
 	    if ($self->create_symbol($2, base => $sym)) {
 		$self->add_symbol($sym, $$obj_ref);
 	    } else {
-		warning(_g('Failed to parse line in %s: %s'), $file, $_);
+		warning(_g('failed to parse line in %s: %s'), $file, $_);
 	    }
 	} elsif (/^(\(.*\))?#include\s+"([^"]+)"/) {
 	    my $tagspec = $1;
@@ -255,7 +263,7 @@ sub parse {
 		$self->create_object($$obj_ref, "$2");
 	    }
 	} else {
-	    warning(_g('Failed to parse a line in %s: %s'), $file, $_);
+	    warning(_g('failed to parse a line in %s: %s'), $file, $_);
 	}
     }
     delete $seen->{$file};
@@ -282,19 +290,19 @@ sub output {
 	my @deps = $self->get_dependencies($soname);
 	my $dep_first = shift @deps;
 	$dep_first =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
-	print $fh "$soname $dep_first\n" if defined $fh;
+	print { $fh } "$soname $dep_first\n" if defined $fh;
 	$res .= "$soname $dep_first\n" if defined wantarray;
 
 	foreach my $dep_next (@deps) {
 	    $dep_next =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
-	    print $fh "| $dep_next\n" if defined $fh;
+	    print { $fh } "| $dep_next\n" if defined $fh;
 	    $res .= "| $dep_next\n" if defined wantarray;
 	}
 	my $f = $self->{objects}{$soname}{fields};
 	foreach my $field (sort keys %{$f}) {
 	    my $value = $f->{$field};
 	    $value =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
-	    print $fh "* $field: $value\n" if defined $fh;
+	    print { $fh } "* $field: $value\n" if defined $fh;
 	    $res .= "* $field: $value\n" if defined wantarray;
 	}
 
@@ -313,14 +321,14 @@ sub output {
 	    next if not $opts{template_mode} and
 	            not $sym->arch_is_concerned($self->get_arch());
 	    # Dump symbol specification. Dump symbol tags only in template mode.
-	    print $fh $sym->get_symbolspec($opts{template_mode}), "\n" if defined $fh;
+	    print { $fh } $sym->get_symbolspec($opts{template_mode}), "\n" if defined $fh;
 	    $res .= $sym->get_symbolspec($opts{template_mode}) . "\n" if defined wantarray;
 	    # Dump pattern matches as comments (if requested)
 	    if ($opts{with_pattern_matches} && $sym->is_pattern()) {
 		for my $match (sort { $a->get_symboltempl() cmp
 		                      $b->get_symboltempl() } $sym->get_pattern_matches())
 		{
-		    print $fh '#MATCH:', $match->get_symbolspec(0), "\n" if defined $fh;
+		    print { $fh } '#MATCH:', $match->get_symbolspec(0), "\n" if defined $fh;
 		    $res .= '#MATCH:' . $match->get_symbolspec(0) . "\n" if defined wantarray;
 		}
 	    }
@@ -385,13 +393,23 @@ sub find_matching_pattern {
 # machinery
 sub merge_symbols {
     my ($self, $object, $minver) = @_;
-    my $soname = $object->{SONAME} || error(_g('cannot merge symbols from objects without SONAME'));
+
+    my $soname = $object->{SONAME};
+    error(_g('cannot merge symbols from objects without SONAME'))
+        unless $soname;
+
+    my %include_groups = ();
+    my $groups = $self->get_field($soname, 'Ignore-Blacklist-Groups');
+    if (defined $groups) {
+        $include_groups{$_} = 1 foreach (split /\s+/, $groups);
+    }
+
     my %dynsyms;
     foreach my $sym ($object->get_exported_dynamic_symbols()) {
         my $name = $sym->{name} . '@' .
                    ($sym->{version} ? $sym->{version} : 'Base');
         my $symobj = $self->lookup_symbol($name, $soname);
-        if (exists $blacklist{$sym->{name}}) {
+        if (symbol_is_blacklisted($sym->{name}, \%include_groups)) {
             next unless (defined $symobj and $symobj->has_tag('ignore-blacklist'));
         }
         $dynsyms{$name} = $sym;
@@ -480,7 +498,8 @@ sub get_smallest_version {
     my ($self, $soname, $dep_id) = @_;
     $dep_id //= 0;
     my $so_object = $self->get_object($soname);
-    return $so_object->{minver_cache}[$dep_id] if(defined($so_object->{minver_cache}[$dep_id]));
+    return $so_object->{minver_cache}[$dep_id]
+        if defined $so_object->{minver_cache}[$dep_id];
     my $minver;
     foreach my $sym ($self->get_symbols($so_object)) {
         next if $dep_id != $sym->{dep_id};
