@@ -1,4 +1,5 @@
 # Copyright © 2008-2011 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2008-2014 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Source::Package::V2;
 
@@ -26,15 +27,17 @@ use Dpkg;
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::File;
+use Dpkg::Path qw(find_command);
 use Dpkg::Compression;
 use Dpkg::Source::Archive;
 use Dpkg::Source::Patch;
-use Dpkg::Exit;
+use Dpkg::Exit qw(push_exit_handler pop_exit_handler);
 use Dpkg::Source::Functions qw(erasedir is_binary fs_time);
 use Dpkg::Vendor qw(run_vendor_hook);
 use Dpkg::Control;
 use Dpkg::Changelog::Parse;
 
+use List::Util qw(first);
 use POSIX qw(:errno_h);
 use Cwd;
 use File::Basename;
@@ -67,6 +70,8 @@ sub init_options {
         unless exists $self->{options}{create_empty_orig};
     $self->{options}{auto_commit} = 0
         unless exists $self->{options}{auto_commit};
+    $self->{options}{ignore_bad_version} = 0
+        unless exists $self->{options}{ignore_bad_version};
 }
 
 sub parse_cmdline_option {
@@ -104,6 +109,9 @@ sub parse_cmdline_option {
     } elsif ($opt =~ /^--auto-commit$/) {
         $self->{options}{auto_commit} = 1;
         return 1;
+    } elsif ($opt =~ /^--ignore-bad-version$/) {
+        $self->{options}{ignore_bad_version} = 1;
+        return 1;
     }
     return 0;
 }
@@ -118,7 +126,7 @@ sub do_extract {
     my $basenamerev = $self->get_basename(1);
 
     my ($tarfile, $debianfile, %origtar, %seen);
-    my $re_ext = $compression_re_file_ext;
+    my $re_ext = compression_get_file_extension_regex();
     foreach my $file ($self->get_files()) {
         (my $uncompressed = $file) =~ s/\.$re_ext$//;
         error(_g('duplicate files in %s source package: %s.*'), 'v2.0',
@@ -153,7 +161,7 @@ sub do_extract {
     # that would be blindly followed when applying the patches
 
     # Extract additional orig tarballs
-    foreach my $subdir (keys %origtar) {
+    foreach my $subdir (sort keys %origtar) {
         my $file = $origtar{$subdir};
         info(_g('unpacking %s'), $file);
         if (-e "$newdirectory/$subdir") {
@@ -201,7 +209,7 @@ sub get_patches {
     my $pd = "$dir/debian/patches";
     my $auto_patch = $self->get_autopatch_name();
     if (-d $pd) {
-        opendir(my $dir_dh, $pd) || syserr(_g('cannot opendir %s'), $pd);
+        opendir(my $dir_dh, $pd) or syserr(_g('cannot opendir %s'), $pd);
         foreach my $patch (sort readdir($dir_dh)) {
             # patches match same rules as run-parts
             next unless $patch =~ /^[\w-]+$/ and -f "$pd/$patch";
@@ -219,9 +227,9 @@ sub apply_patches {
     my @patches = $self->get_patches($dir, %opts);
     return unless scalar(@patches);
     my $applied = File::Spec->catfile($dir, 'debian', 'patches', '.dpkg-source-applied');
-    open(my $applied_fh, '>', $applied) ||
-        syserr(_g('cannot write %s'), $applied);
-    print $applied_fh "# During $opts{usage}\n";
+    open(my $applied_fh, '>', $applied)
+        or syserr(_g('cannot write %s'), $applied);
+    print { $applied_fh } "# During $opts{usage}\n";
     my $timestamp = fs_time($applied);
     foreach my $patch ($self->get_patches($dir, %opts)) {
         my $path = File::Spec->catfile($dir, 'debian', 'patches', $patch);
@@ -230,7 +238,7 @@ sub apply_patches {
         $patch_obj->apply($dir, force_timestamp => 1,
                           timestamp => $timestamp,
                           add_options => [ '-E' ]);
-        print $applied_fh "$patch\n";
+        print { $applied_fh } "$patch\n";
     }
     close($applied_fh);
 }
@@ -280,8 +288,8 @@ sub after_build {
     my $applied = File::Spec->catfile($dir, 'debian', 'patches', '.dpkg-source-applied');
     my $reason = '';
     if (-e $applied) {
-        open(my $applied_fh, '<', $applied) ||
-            syserr(_g('cannot read %s'), $applied);
+        open(my $applied_fh, '<', $applied)
+            or syserr(_g('cannot read %s'), $applied);
         $reason = <$applied_fh>;
         close($applied_fh);
     }
@@ -295,8 +303,8 @@ sub after_build {
 sub prepare_build {
     my ($self, $dir) = @_;
     $self->{diff_options} = {
-        diff_ignore_regexp => $self->{options}{diff_ignore_regexp} .
-                              '|(^|/)debian/patches/.dpkg-source-applied$',
+        diff_ignore_regex => $self->{options}{diff_ignore_regex} .
+                             '|(^|/)debian/patches/.dpkg-source-applied$',
         include_removal => $self->{options}{include_removal},
         include_timestamp => $self->{options}{include_timestamp},
         use_dev_null => 1,
@@ -309,7 +317,8 @@ sub prepare_build {
         # No main orig.tar, create a dummy one
         my $filename = $self->get_basename() . '.orig.tar.' .
                        $self->{options}{comp_ext};
-        my $tar = Dpkg::Source::Archive->new(filename => $filename);
+        my $tar = Dpkg::Source::Archive->new(filename => $filename,
+                                             compression_level => $self->{options}{comp_level});
         $tar->create();
         $tar->finish();
     }
@@ -332,9 +341,10 @@ sub generate_patch {
 
     # Identify original tarballs
     my ($tarfile, %origtar);
+    my $comp_ext_regex = compression_get_file_extension_regex();
     my @origtarballs;
     foreach (sort $self->find_original_tarballs()) {
-        if (/\.orig\.tar\.$compression_re_file_ext$/) {
+        if (/\.orig\.tar\.$comp_ext_regex$/) {
             if (defined($tarfile)) {
                 error(_g('several orig.tar files found (%s and %s) but only ' .
                          'one is allowed'), $tarfile, $_);
@@ -342,7 +352,7 @@ sub generate_patch {
             $tarfile = $_;
             push @origtarballs, $_;
             $self->add_file($_);
-        } elsif (/\.orig-([[:alnum:]-]+)\.tar\.$compression_re_file_ext$/) {
+        } elsif (/\.orig-([[:alnum:]-]+)\.tar\.$comp_ext_regex$/) {
             $origtar{$1} = $_;
             push @origtarballs, $_;
             $self->add_file($_);
@@ -359,7 +369,7 @@ sub generate_patch {
 
     # Unpack a second copy for comparison
     my $tmp = tempdir("$dirname.orig.XXXXXX", DIR => $updir);
-    push @Dpkg::Exit::handlers, sub { erasedir($tmp) };
+    push_exit_handler(sub { erasedir($tmp) });
 
     # Extract main tarball
     my $tar = Dpkg::Source::Archive->new(filename => $tarfile);
@@ -384,7 +394,7 @@ sub generate_patch {
     # Create a patch
     my ($difffh, $tmpdiff) = tempfile($self->get_basename(1) . '.diff.XXXXXX',
                                       DIR => File::Spec->tmpdir(), UNLINK => 0);
-    push @Dpkg::Exit::handlers, sub { unlink($tmpdiff) };
+    push_exit_handler(sub { unlink($tmpdiff) });
     my $diff = Dpkg::Source::Patch->new(filename => $tmpdiff,
                                         compression => 'none');
     $diff->create();
@@ -412,8 +422,8 @@ sub generate_patch {
 
     # Remove the temporary directory
     erasedir($tmp);
-    pop @Dpkg::Exit::handlers;
-    pop @Dpkg::Exit::handlers;
+    pop_exit_handler();
+    pop_exit_handler();
 
     return $tmpdiff;
 }
@@ -459,13 +469,13 @@ sub do_build {
         my $reldir = File::Spec->abs2rel($File::Find::dir, $dir);
         my $cwd = getcwd();
         # Apply the pattern both from the top dir and from the inspected dir
-        chdir($dir) || syserr(_g("unable to chdir to `%s'"), $dir);
+        chdir($dir) or syserr(_g("unable to chdir to `%s'"), $dir);
         $exclude{$_} = 1 foreach glob($tar_ignore_glob);
-        chdir($cwd) || syserr(_g("unable to chdir to `%s'"), $cwd);
-        chdir($File::Find::dir) ||
-            syserr(_g("unable to chdir to `%s'"), $File::Find::dir);
+        chdir($cwd) or syserr(_g("unable to chdir to `%s'"), $cwd);
+        chdir($File::Find::dir)
+            or syserr(_g("unable to chdir to `%s'"), $File::Find::dir);
         $exclude{$_} = 1 foreach glob($tar_ignore_glob);
-        chdir($cwd) || syserr(_g("unable to chdir to `%s'"), $cwd);
+        chdir($cwd) or syserr(_g("unable to chdir to `%s'"), $cwd);
         my @result;
         foreach my $fn (@_) {
             unless (exists $exclude{$fn} or exists $exclude{"$reldir/$fn"}) {
@@ -512,7 +522,7 @@ sub do_build {
         error(_g('aborting due to unexpected upstream changes, see %s'),
               $tmpdiff);
     }
-    push @Dpkg::Exit::handlers, sub { unlink($tmpdiff) };
+    push_exit_handler(sub { unlink($tmpdiff) });
     $binaryfiles->update_debian_source_include_binaries() if $include_binaries;
 
     # Install the diff as the new autopatch
@@ -524,13 +534,14 @@ sub do_build {
              $autopatch) if -e $autopatch;
         rmdir(File::Spec->catdir($dir, 'debian', 'patches')); # No check on purpose
     }
-    unlink($tmpdiff) || syserr(_g('cannot remove %s'), $tmpdiff);
-    pop @Dpkg::Exit::handlers;
+    unlink($tmpdiff) or syserr(_g('cannot remove %s'), $tmpdiff);
+    pop_exit_handler();
 
     # Create the debian.tar
     my $debianfile = "$basenamerev.debian.tar." . $self->{options}{comp_ext};
     info(_g('building %s in %s'), $sourcepackage, $debianfile);
-    my $tar = Dpkg::Source::Archive->new(filename => $debianfile);
+    my $tar = Dpkg::Source::Archive->new(filename => $debianfile,
+                                         compression_level => $self->{options}{comp_level});
     $tar->create(options => \@tar_ignore, chdir => $dir);
     $tar->add_directory('debian');
     foreach my $binary ($binaryfiles->get_seen_binaries()) {
@@ -549,7 +560,7 @@ sub get_patch_header {
     }
     my $text;
     if (-f $ph) {
-        open(my $ph_fh, '<', $ph) || syserr(_g('cannot read %s'), $ph);
+        open(my $ph_fh, '<', $ph) or syserr(_g('cannot read %s'), $ph);
         $text = file_slurp($ph_fh);
         close($ph_fh);
         return $text;
@@ -588,17 +599,17 @@ sub register_patch {
     my ($self, $dir, $patch_file, $patch_name) = @_;
     my $patch = File::Spec->catfile($dir, 'debian', 'patches', $patch_name);
     if (-s $patch_file) {
-        copy($patch_file, $patch) ||
-            syserr(_g('failed to copy %s to %s'), $patch_file, $patch);
-        chmod(0666 & ~ umask(), $patch) ||
-                syserr(_g("unable to change permission of `%s'"), $patch);
+        copy($patch_file, $patch)
+            or syserr(_g('failed to copy %s to %s'), $patch_file, $patch);
+        chmod(0666 & ~ umask(), $patch)
+            or syserr(_g("unable to change permission of `%s'"), $patch);
         my $applied = File::Spec->catfile($dir, 'debian', 'patches', '.dpkg-source-applied');
-        open(my $applied_fh, '>>', $applied) ||
-            syserr(_g('cannot write %s'), $applied);
-        print $applied_fh "$patch\n";
-        close($applied_fh) || syserr(_g('cannot close %s'), $applied);
+        open(my $applied_fh, '>>', $applied)
+            or syserr(_g('cannot write %s'), $applied);
+        print { $applied_fh } "$patch\n";
+        close($applied_fh) or syserr(_g('cannot close %s'), $applied);
     } elsif (-e $patch) {
-        unlink($patch) || syserr(_g('cannot remove %s'), $patch);
+        unlink($patch) or syserr(_g('cannot remove %s'), $patch);
     }
     return $patch;
 }
@@ -643,24 +654,30 @@ sub do_commit {
                                          usage => 'commit');
         $binaryfiles->update_debian_source_include_binaries();
     }
-    push @Dpkg::Exit::handlers, sub { unlink($tmpdiff) };
+    push_exit_handler(sub { unlink($tmpdiff) });
     unless (-s $tmpdiff) {
-        unlink($tmpdiff) || syserr(_g('cannot remove %s'), $tmpdiff);
+        unlink($tmpdiff) or syserr(_g('cannot remove %s'), $tmpdiff);
         info(_g('there are no local changes to record'));
         return;
     }
     while (_is_bad_patch_name($dir, $patch_name)) {
         # Ask the patch name interactively
-        print STDOUT _g('Enter the desired patch name: ');
+        print _g('Enter the desired patch name: ');
         chomp($patch_name = <STDIN>);
         $patch_name =~ s/\s+/-/g;
         $patch_name =~ s/\///g;
     }
     mkpath(File::Spec->catdir($dir, 'debian', 'patches'));
     my $patch = $self->register_patch($dir, $tmpdiff, $patch_name);
-    system('sensible-editor', $patch);
-    unlink($tmpdiff) || syserr(_g('cannot remove %s'), $tmpdiff);
-    pop @Dpkg::Exit::handlers;
+    my @editors = ('sensible-editor', $ENV{VISUAL}, $ENV{EDITOR}, 'vi');
+    my $editor = first { defined $_ and find_command($_) } @editors;
+    if (not $editor) {
+        error(_g('cannot find an editor'));
+    }
+    system($editor, $patch);
+    subprocerr($editor) if $?;
+    unlink($tmpdiff) or syserr(_g('cannot remove %s'), $tmpdiff);
+    pop_exit_handler();
     info(_g('local changes have been recorded in a new patch: %s'), $patch);
 }
 
@@ -697,8 +714,8 @@ sub load_allowed_binaries {
     my ($self) = @_;
     my $incbin_file = $self->{include_binaries_path};
     if (-f $incbin_file) {
-        open(my $incbin_fh, '<', $incbin_file) ||
-            syserr(_g('cannot read %s'), $incbin_file);
+        open(my $incbin_fh, '<', $incbin_file)
+            or syserr(_g('cannot read %s'), $incbin_file);
         while (defined($_ = <$incbin_fh>)) {
             chomp; s/^\s*//; s/\s*$//;
             next if /^#/ or /^$/;
@@ -722,10 +739,10 @@ sub update_debian_source_include_binaries {
 
     my $incbin_file = $self->{include_binaries_path};
     mkpath(File::Spec->catdir($self->{dir}, 'debian', 'source'));
-    open(my $incbin_fh, '>>', $incbin_file) ||
-        syserr(_g('cannot write %s'), $incbin_file);
+    open(my $incbin_fh, '>>', $incbin_file)
+        or syserr(_g('cannot write %s'), $incbin_file);
     foreach my $binary (@unknown_binaries) {
-        print $incbin_fh "$binary\n";
+        print { $incbin_fh } "$binary\n";
         info(_g('adding %s to %s'), $binary, 'debian/source/include-binaries');
         $self->{allowed_binaries}{$binary} = 1;
     }
