@@ -11,7 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Source::Patch;
 
@@ -107,6 +107,8 @@ sub add_diff_file {
     # Check diff and write it in patch file
     my $difflinefound = 0;
     my $binary = 0;
+    local $_;
+
     while (<$diffgen>) {
         if (m/^(?:binary|[^-+\@ ].*\bdiffer\b)/i) {
             $binary = 1;
@@ -125,7 +127,7 @@ sub add_diff_file {
 	    $self->print(*$self->{header}) or syserr(_g('failed to write'));
 	    *$self->{empty} = 0;
 	}
-        print $self $_ || syserr(_g('failed to write'));
+        print { $self } $_ or syserr(_g('failed to write'));
     }
     close($diffgen) or syserr('close on diff pipe');
     wait_child($diff_pid, nocheck => 1,
@@ -149,8 +151,8 @@ sub add_diff_directory {
     my $diff_ignore;
     if ($opts{diff_ignore_func}) {
         $diff_ignore = $opts{diff_ignore_func};
-    } elsif ($opts{diff_ignore_regexp}) {
-        $diff_ignore = sub { return $_[0] =~ /$opts{diff_ignore_regexp}/o };
+    } elsif ($opts{diff_ignore_regex}) {
+        $diff_ignore = sub { return $_[0] =~ /$opts{diff_ignore_regex}/o };
     } else {
         $diff_ignore = sub { return 0 };
     }
@@ -161,7 +163,7 @@ sub add_diff_directory {
         my $fn = (length > length($new)) ? substr($_, length($new) + 1) : '.';
         return if &$diff_ignore($fn);
         $files_in_new{$fn} = 1;
-        lstat("$new/$fn") || syserr(_g('cannot stat file %s'), "$new/$fn");
+        lstat("$new/$fn") or syserr(_g('cannot stat file %s'), "$new/$fn");
         my $mode = S_IMODE((lstat(_))[2]);
         my $size = (lstat(_))[7];
         if (-l _) {
@@ -169,18 +171,23 @@ sub add_diff_directory {
                 $self->_fail_not_same_type("$old/$fn", "$new/$fn");
                 return;
             }
-            defined(my $n = readlink("$new/$fn")) ||
+            my $n = readlink("$new/$fn");
+            unless (defined $n) {
                 syserr(_g('cannot read link %s'), "$new/$fn");
-            defined(my $n2 = readlink("$old/$fn")) ||
+            }
+            my $n2 = readlink("$old/$fn");
+            unless (defined $n2) {
                 syserr(_g('cannot read link %s'), "$old/$fn");
+            }
             unless ($n eq $n2) {
                 $self->_fail_not_same_type("$old/$fn", "$new/$fn");
             }
         } elsif (-f _) {
             my $old_file = "$old/$fn";
             if (not lstat("$old/$fn")) {
-                $! == ENOENT ||
+                if ($! != ENOENT) {
                     syserr(_g('cannot stat file %s'), "$old/$fn");
+                }
                 $old_file = '/dev/null';
             } elsif (not -f _) {
                 $self->_fail_not_same_type("$old/$fn", "$new/$fn");
@@ -202,8 +209,9 @@ sub add_diff_directory {
                 _g('device or socket is not allowed'));
         } elsif (-d _) {
             if (not lstat("$old/$fn")) {
-                $! == ENOENT ||
+                if ($! != ENOENT) {
                     syserr(_g('cannot stat file %s'), "$old/$fn");
+                }
             } elsif (not -d _) {
                 $self->_fail_not_same_type("$old/$fn", "$new/$fn");
             }
@@ -215,13 +223,13 @@ sub add_diff_directory {
         my $fn = (length > length($old)) ? substr($_, length($old) + 1) : '.';
         return if &$diff_ignore($fn);
         return if $files_in_new{$fn};
-        lstat("$old/$fn") || syserr(_g('cannot stat file %s'), "$old/$fn");
+        lstat("$old/$fn") or syserr(_g('cannot stat file %s'), "$old/$fn");
         if (-f _) {
             if ($inc_removal) {
                 push @diff_files, [$fn, 0, 0, "$old/$fn", '/dev/null',
                                    "$basedir.orig/$fn", '/dev/null'];
             } else {
-                warning(_g('ignoring deletion of file %s'), $fn);
+                warning(_g('ignoring deletion of file %s, use --include-removal to override'), $fn);
             }
         } elsif (-d _) {
             warning(_g('ignoring deletion of directory %s'), $fn);
@@ -289,7 +297,7 @@ sub add_diff_directory {
 
 sub finish {
     my ($self) = @_;
-    close($self) || syserr(_g('cannot close %s'), $self->get_filename());
+    close($self) or syserr(_g('cannot close %s'), $self->get_filename());
     return not *$self->{errors};
 }
 
@@ -324,14 +332,23 @@ sub _getline {
     return $line;
 }
 
-# Strip timestamp
-sub _strip_ts {
-    my $header = shift;
+# Fetch the header filename ignoring the optional timestamp
+sub _fetch_filename {
+    my ($diff, $header) = @_;
+
+    # Strip any leading spaces.
+    $header =~ s/^\s+//;
+
+    # Is it a C-style string?
+    if ($header =~ m/^"/) {
+        error(_g('diff %s patches file with C-style encoded filename'), $diff);
+    } else {
+        # Tab is the official separator, it's always used when
+        # filename contain spaces. Try it first, otherwise strip on space
+        # if there's no tab
+        $header =~ s/\s.*// unless $header =~ s/\t.*//;
+    }
 
-    # Tab is the official separator, it's always used when
-    # filename contain spaces. Try it first, otherwise strip on space
-    # if there's no tab
-    $header =~ s/\s.*// unless ($header =~ s/\t.*//);
     return $header;
 }
 
@@ -388,7 +405,7 @@ sub analyze {
 	my (%path, %fn);
 	# skip comments leading up to patch (if any)
 	while (1) {
-	    if (/^--- /) {
+	    if (/^(?:--- |\+\+\+ |@@ -)/) {
 		last;
 	    } else {
 		$patch_header .= "$_\n";
@@ -397,10 +414,10 @@ sub analyze {
 	}
 	$diff_count++;
 	# read file header (---/+++ pair)
-	unless(s/^--- //) {
+	unless (s/^--- //) {
 	    error(_g("expected ^--- in line %d of diff `%s'"), $., $diff);
 	}
-        $path{old} = $_ = _strip_ts($_);
+	$path{old} = $_ = _fetch_filename($diff, $_);
 	$fn{old} = $_ if $_ ne '/dev/null' and s{^[^/]*/+}{$destdir/};
 	if (/\.dpkg-orig$/) {
 	    error(_g("diff `%s' patches file with name ending .dpkg-orig"), $diff);
@@ -412,7 +429,7 @@ sub analyze {
 	unless (s/^\+\+\+ //) {
 	    error(_g("line after --- isn't as expected in diff `%s' (line %d)"), $diff, $.);
 	}
-        $path{new} = $_ = _strip_ts($_);
+	$path{new} = $_ = _fetch_filename($diff, $_);
 	$fn{new} = $_ if $_ ne '/dev/null' and s{^[^/]*/+}{$destdir/};
 
 	unless (defined $fn{old} or defined $fn{new}) {
@@ -472,7 +489,7 @@ sub analyze {
 	while (defined($_ = _getline($self))) {
 	    # read hunk header (@@)
 	    next if /^\\ /;
-	    last unless (/^@@ -\d+(,(\d+))? \+\d+(,(\d+))? @\@( .*)?$/);
+	    last unless (/^@@ -\d+(,(\d+))? \+\d+(,(\d+))? @\@(?: .*)?$/);
 	    my ($olines, $nlines) = ($1 ? $2 : 1, $3 ? $4 : 1);
 	    # read hunk
 	    while ($olines || $nlines) {
@@ -497,7 +514,7 @@ sub analyze {
 	    }
 	    $hunk++;
 	}
-	unless($hunk) {
+	unless ($hunk) {
 	    error(_g("expected ^\@\@ at line %d of diff `%s'"), $., $diff);
 	}
     }
@@ -551,8 +568,8 @@ sub apply {
 	error_to_string => \$stderr,
     );
     if ($?) {
-	print STDOUT $stdout;
-	print STDERR $stderr;
+	print { *STDOUT } $stdout;
+	print { *STDERR } $stderr;
 	subprocerr('LC_ALL=C patch ' . join(' ', @{$opts{options}}) .
 	           ' < ' . $self->get_filename());
     }
@@ -564,18 +581,18 @@ sub apply {
     $now ||= fs_time($files[0]) if $opts{force_timestamp} && scalar @files;
     foreach my $fn (@files) {
 	if ($opts{force_timestamp}) {
-	    utime($now, $now, $fn) || $! == ENOENT ||
-		syserr(_g('cannot change timestamp for %s'), $fn);
+	    utime($now, $now, $fn) or $! == ENOENT
+		or syserr(_g('cannot change timestamp for %s'), $fn);
 	}
 	if ($opts{remove_backup}) {
 	    $fn .= '.dpkg-orig';
-	    unlink($fn) || syserr(_g('remove patch backup file %s'), $fn);
+	    unlink($fn) or syserr(_g('remove patch backup file %s'), $fn);
 	}
     }
     return $analysis;
 }
 
-# Verify if check will work...
+# Verify if check will work...
 sub check_apply {
     my ($self, $destdir, %opts) = @_;
     # Set default values to options
