@@ -11,17 +11,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Source::Quilt;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
+use Dpkg::Util qw(:list);
 use Dpkg::Source::Patch;
 use Dpkg::Source::Functions qw(erasedir fs_time);
 use Dpkg::Vendor qw(get_current_vendor);
@@ -56,7 +57,7 @@ sub setup_db {
     my $file = $self->get_db_file('.version');
     if (not -e $file) {
         open(my $version_fh, '>', $file) or syserr(_g('cannot write %s'), $file);
-        print $version_fh "2\n";
+        print { $version_fh } "2\n";
         close($version_fh);
     }
     # The files below are used by quilt to know where patches are stored
@@ -65,7 +66,7 @@ sub setup_db {
     $file = $self->get_db_file('.quilt_patches');
     if (not -e $file) {
         open(my $qpatch_fh, '>', $file) or syserr(_g('cannot write %s'), $file);
-        print $qpatch_fh "debian/patches\n";
+        print { $qpatch_fh } "debian/patches\n";
         close($qpatch_fh);
     }
     $file = $self->get_db_file('.quilt_series');
@@ -73,7 +74,7 @@ sub setup_db {
         open(my $qseries_fh, '>', $file) or syserr(_g('cannot write %s'), $file);
         my $series = $self->get_series_file();
         $series = (File::Spec->splitpath($series))[2];
-        print $qseries_fh "$series\n";
+        print { $qseries_fh } "$series\n";
         close($qseries_fh);
     }
 }
@@ -85,17 +86,12 @@ sub load_db {
     $self->{applied_patches} = [ $self->read_patch_list($pc_applied) ];
 }
 
-sub write_db {
+sub save_db {
     my ($self) = @_;
 
     $self->setup_db();
     my $pc_applied = $self->get_db_file('applied-patches');
-    open(my $applied_fh, '>', $pc_applied) or
-        syserr(_g('cannot write %s'), $pc_applied);
-    foreach my $patch (@{$self->{applied_patches}}) {
-        print $applied_fh "$patch\n";
-    }
-    close($applied_fh);
+    $self->write_patch_list($pc_applied, $self->{applied_patches});
 }
 
 sub load_series {
@@ -122,6 +118,42 @@ sub top {
     return;
 }
 
+sub register {
+    my ($self, $patch_name) = @_;
+
+    return if any { $_ eq $patch_name } @{$self->{series}};
+
+    # Add patch to series files.
+    $self->setup_db();
+    $self->_file_add_line($self->get_series_file(), $patch_name);
+    $self->_file_add_line($self->get_db_file('applied-patches'), $patch_name);
+    $self->load_db();
+    $self->load_series();
+
+    # Ensure quilt meta-data is created and in sync with some trickery:
+    # Reverse-apply the patch, drop .pc/$patch, and re-apply it with the
+    # correct options to recreate the backup files.
+    $self->pop(reverse_apply => 1);
+    $self->push();
+}
+
+sub unregister {
+    my ($self, $patch_name) = @_;
+
+    return if none { $_ eq $patch_name } @{$self->{series}};
+
+    my $series = $self->get_series_file();
+
+    $self->_file_drop_line($series, $patch_name);
+    $self->_file_drop_line($self->get_db_file('applied-patches'), $patch_name);
+    erasedir($self->get_db_file($patch_name));
+    $self->load_db();
+    $self->load_series();
+
+    # Clean up empty series.
+    unlink $series if -z $series;
+}
+
 sub next {
     my ($self) = @_;
     my $count_applied = scalar @{$self->{applied_patches}};
@@ -159,7 +191,7 @@ sub push {
         die $@;
     }
     CORE::push @{$self->{applied_patches}}, $patch;
-    $self->write_db();
+    $self->save_db();
 }
 
 sub pop {
@@ -190,14 +222,14 @@ sub pop {
 
     erasedir($backup_dir);
     pop @{$self->{applied_patches}};
-    $self->write_db();
+    $self->save_db();
 }
 
 sub get_db_version {
     my ($self) = @_;
     my $pc_ver = $self->get_db_file('.version');
     if (-f $pc_ver) {
-        open(my $ver_fh, '<', $pc_ver) || syserr(_g('cannot read %s'), $pc_ver);
+        open(my $ver_fh, '<', $pc_ver) or syserr(_g('cannot read %s'), $pc_ver);
         my $version = <$ver_fh>;
         chomp $version;
         close($ver_fh);
@@ -251,15 +283,47 @@ sub get_patch_dir {
 
 ## METHODS BELOW ARE INTERNAL ##
 
+sub _file_load {
+    my ($self, $file) = @_;
+
+    open my $file_fh, '<', $file or syserr(_g('cannot read %s'), $file);
+    my @lines = <$file_fh>;
+    close $file_fh;
+
+    return @lines;
+}
+
+sub _file_add_line {
+    my ($self, $file, $line) = @_;
+
+    my @lines;
+    @lines = $self->_file_load($file) if -f $file;
+    CORE::push @lines, $line;
+    chomp @lines;
+
+    open my $file_fh, '>', $file or syserr(_g('cannot write %s'), $file);
+    print { $file_fh } "$_\n" foreach @lines;
+    close $file_fh;
+}
+
+sub _file_drop_line {
+    my ($self, $file, $re) = @_;
+
+    my @lines = $self->_file_load($file);
+    open my $file_fh, '>', $file or syserr(_g('cannot write %s'), $file);
+    print { $file_fh } $_ foreach grep { not /^\Q$re\E\s*$/ } @lines;
+    close $file_fh;
+}
+
 sub read_patch_list {
     my ($self, $file, %opts) = @_;
     return () if not defined $file or not -f $file;
     $opts{warn_options} //= 0;
     my @patches;
-    open(my $series_fh, '<' , $file) || syserr(_g('cannot read %s'), $file);
+    open(my $series_fh, '<' , $file) or syserr(_g('cannot read %s'), $file);
     while (defined($_ = <$series_fh>)) {
         chomp; s/^\s+//; s/\s+$//; # Strip leading/trailing spaces
-        s/(^|\s+)#.*$//; # Strip comment
+        s/(?:^|\s+)#.*$//; # Strip comment
         next unless $_;
         if (/^(\S+)\s+(.*)$/) {
             $_ = $1;
@@ -277,6 +341,16 @@ sub read_patch_list {
     return @patches;
 }
 
+sub write_patch_list {
+    my ($self, $series, $patches) = @_;
+
+    open my $series_fh, '>', $series or syserr(_g('cannot write %s'), $series);
+    foreach my $patch (@{$patches}) {
+        print { $series_fh } "$patch\n";
+    }
+    close $series_fh;
+}
+
 sub restore_quilt_backup_files {
     my ($self, $patch, %opts) = @_;
     my $patch_dir = $self->get_db_file($patch);
@@ -292,10 +366,10 @@ sub restore_quilt_backup_files {
                 unlink($target);
                 make_path(dirname($target));
                 unless (link($_, $target)) {
-                    copy($_, $target) ||
-                        syserr(_g('failed to copy %s to %s'), $_, $target);
-                    chmod((stat(_))[2], $target) ||
-                        syserr(_g("unable to change permission of `%s'"), $target);
+                    copy($_, $target)
+                        or syserr(_g('failed to copy %s to %s'), $_, $target);
+                    chmod((stat(_))[2], $target)
+                        or syserr(_g("unable to change permission of `%s'"), $target);
                 }
             } else {
                 # empty files are "backups" for new files that patch created
