@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Substvars;
 
@@ -169,13 +169,16 @@ the filehandle in error messages.
 
 sub parse {
     my ($self, $fh, $varlistfile) = @_;
+    local $_;
+
     binmode($fh);
     while (<$fh>) {
 	next if m/^\s*\#/ || !m/\S/;
 	s/\s*\n$//;
-	m/^(\w[-:0-9A-Za-z]*)\=(.*)$/ ||
+	if (! m/^(\w[-:0-9A-Za-z]*)\=(.*)$/) {
 	    error(_g('bad line in substvars file %s at line %d'),
 		  $varlistfile, $.);
+	}
 	$self->{vars}{$1} = $2;
     }
 }
@@ -244,9 +247,10 @@ sub substvars {
         # reset the recursive counter.
         $count = 0 if (length($3) < length($rhs));
 
-        $count < $maxsubsts ||
+        if ($count >= $maxsubsts) {
             error($opts{msg_prefix} .
 	          _g("too many substitutions - recursive ? - in \`%s'"), $v);
+        }
         $lhs = $1; $vn = $2; $rhs = $3;
         if (defined($self->{vars}{$vn})) {
             $v = $lhs . $self->{vars}{$vn} . $rhs;
@@ -318,7 +322,7 @@ sub output {
     foreach my $vn (sort keys %{$self->{vars}}) {
 	next if /^(?:(?:dpkg|source|binary):(?:Source-)?Version|Space|Tab|Newline|Arch|Source-Version|F:.+)$/;
 	my $line = "$vn=" . $self->{vars}{$vn} . "\n";
-	print $fh $line if defined $fh;
+	print { $fh } $line if defined $fh;
 	$str .= $line;
     }
     return $str;
@@ -326,6 +330,21 @@ sub output {
 
 =back
 
+=head1 CHANGES
+
+=head2 Version 1.02
+
+New argument: Accept a $binaryversion in $s->set_version_substvars(),
+passing a single argument is still supported.
+
+New method: $s->mark_as_used().
+
+Deprecated method: $s->no_warn(), use $s->mark_as_used() instead.
+
+=head2 Version 1.01
+
+New method: $s->set_as_used().
+
 =head1 AUTHOR
 
 Raphaël Hertzog <hertzog@debian.org>.
