@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Vendor::Debian;
 
@@ -115,21 +115,21 @@ sub add_hardening_flags {
     }
 
     # Mask features that are not available on certain architectures.
-    if ($os !~ /^(linux|knetbsd|hurd)$/ or
-	$cpu =~ /^(hppa|mips|mipsel|avr32)$/) {
+    if ($os !~ /^(?:linux|knetbsd|hurd)$/ or
+	$cpu =~ /^(?:hppa|mips|mipsel|avr32)$/) {
 	# Disabled on non-linux/knetbsd/hurd (see #430455 and #586215).
 	# Disabled on hppa, mips/mipsel (#532821), avr32
 	#  (#574716).
 	$use_feature{pie} = 0;
     }
-    if ($cpu =~ /^(ia64|alpha|mips|mipsel|hppa|arm64)$/ or $arch eq 'arm') {
+    if ($cpu =~ /^(?:ia64|alpha|mips|mipsel|hppa|arm64)$/ or $arch eq 'arm') {
 	# Stack protector disabled on ia64, alpha, arm64, mips, mipsel, hppa.
 	#   "warning: -fstack-protector not supported for this target"
 	# Stack protector disabled on arm (ok on armel).
 	#   compiler supports it incorrectly (leads to SEGV)
 	$use_feature{stackprotector} = 0;
     }
-    if ($cpu =~ /^(ia64|hppa|avr32)$/) {
+    if ($cpu =~ /^(?:ia64|hppa|avr32)$/) {
 	# relro not implemented on ia64, hppa, avr32.
 	$use_feature{relro} = 0;
     }
@@ -150,6 +150,10 @@ sub add_hardening_flags {
     # PIE
     if ($use_feature{pie}) {
 	$flags->append('CFLAGS', '-fPIE');
+	$flags->append('OBJCFLAGS', '-fPIE');
+	$flags->append('OBJCXXFLAGS', '-fPIE');
+	$flags->append('FFLAGS', '-fPIE');
+	$flags->append('FCFLAGS', '-fPIE');
 	$flags->append('CXXFLAGS', '-fPIE');
 	$flags->append('GCJFLAGS', '-fPIE');
 	$flags->append('LDFLAGS', '-fPIE -pie');
@@ -158,6 +162,10 @@ sub add_hardening_flags {
     # Stack protector
     if ($use_feature{stackprotector}) {
 	$flags->append('CFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
+	$flags->append('OBJCFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
+	$flags->append('OBJCXXFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
+	$flags->append('FFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
+	$flags->append('FCFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
 	$flags->append('CXXFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
 	$flags->append('GCJFLAGS', '-fstack-protector --param=ssp-buffer-size=4');
     }
@@ -171,6 +179,8 @@ sub add_hardening_flags {
     if ($use_feature{format}) {
 	$flags->append('CFLAGS', '-Wformat -Werror=format-security');
 	$flags->append('CXXFLAGS', '-Wformat -Werror=format-security');
+	$flags->append('OBJCFLAGS', '-Wformat -Werror=format-security');
+	$flags->append('OBJCXXFLAGS', '-Wformat -Werror=format-security');
     }
 
     # Read-only Relocations
