@@ -17,7 +17,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -49,11 +49,11 @@ sub usage {
 'Options:
   -a<debian-arch>    set current Debian architecture.
   -t<gnu-system>     set current GNU system type.
-  -L                 list valid architectures.
   -f                 force flag (override variables set in environment).')
     . "\n\n" . _g(
 'Commands:
   -l                 list variables (default).
+  -L                 list valid architectures.
   -e<debian-arch>    compare with current Debian architecture.
   -i<arch-alias>     check if current Debian architecture is <arch-alias>.
   -q<variable>       prints only the value of <variable>.
@@ -122,16 +122,16 @@ sub action_needs($) {
 
 while (@ARGV) {
     $_=shift(@ARGV);
-    if (m/^-a/) {
-	$req_host_arch = "$'";
-    } elsif (m/^-t/) {
-	$req_host_gnu_type = "$'";
-    } elsif (m/^-e/) {
-	$req_eq_arch = "$'";
+    if (m/^-a/p) {
+	$req_host_arch = ${^POSTMATCH};
+    } elsif (m/^-t/p) {
+	$req_host_gnu_type = ${^POSTMATCH};
+    } elsif (m/^-e/p) {
+	$req_eq_arch = ${^POSTMATCH};
 	$req_vars = $arch_vars{DEB_HOST_ARCH};
 	$action = 'e';
-    } elsif (m/^-i/) {
-	$req_is_arch = "$'";
+    } elsif (m/^-i/p) {
+	$req_is_arch = ${^POSTMATCH};
 	$req_vars = $arch_vars{DEB_HOST_ARCH};
 	$action = 'i';
     } elsif (m/^-u$/) {
@@ -142,8 +142,8 @@ while (@ARGV) {
 	$action =~ s/^-//;
     } elsif (m/^-f$/) {
         $force=1;
-    } elsif (m/^-q/) {
-	my $varname = "$'";
+    } elsif (m/^-q/p) {
+	my $varname = ${^POSTMATCH};
 	error(_g('%s is not a supported variable name'), $varname)
 	    unless (exists $arch_vars{$varname});
 	$req_variable_to_print = "$varname";
@@ -155,7 +155,7 @@ while (@ARGV) {
     } elsif (m/^-L$/) {
         list_arches();
         exit unless @ARGV;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-(?:\?|-help)$/) {
         usage();
        exit 0;
     } elsif (m/^--version$/) {
