@@ -18,11 +18,12 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
 
+use Carp;
 use Cwd;
 use File::Basename;
 use POSIX qw(:sys_wait_h);
@@ -31,8 +32,11 @@ use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::BuildOptions;
+use Dpkg::BuildProfiles qw(set_build_profiles);
 use Dpkg::Compression;
+use Dpkg::Checksums;
 use Dpkg::Version;
+use Dpkg::Control;
 use Dpkg::Changelog::Parse;
 use Dpkg::Path qw(find_command);
 use Dpkg::IPC;
@@ -57,19 +61,29 @@ sub usage {
   -b             binary-only, do not build source.
   -B             binary-only, no arch-indep files.
   -A             binary-only, only arch-indep files.
-  -S             source only, no binary files.
+  -S             source-only, no binary files.
   -nc            do not clean source tree (implies -b).
   -tc            clean source tree when finished.
   -D (default)   check build dependencies and conflicts.
   -d             do not check build dependencies and conflicts.
+  -P<profiles>   assume given build profiles as active (comma-separated list).
   -R<rules>      rules file to execute (default is debian/rules).
   -T<target>     call debian/rules <target> with the proper environment.
       --as-root  ensure -T calls the target with root rights.
   -j[<number>]   specify jobs to run simultaneously (passed to <rules>).
   -r<gain-root-command>
                  command to gain root privileges (default is fakeroot).
+  --check-command=<check-command>
+                 command to check the .changes file (no default).
+  --check-option=<opt>
+                 pass <opt> to <check-command>.
+  --hook-<hook-name>=<hook-command>
+                 set <hook-command> as the hook <hook-name>, known hooks:
+                   init preclean source build binary changes postclean
+                   check sign done
   -p<sign-command>
-                 command to sign .dsc and/or .changes files (default is gpg).
+                 command to sign .dsc and/or .changes files
+                   (default is gpg2 or gpg).
   -k<keyid>      the key to use for signing.
   -ap            add pause before starting signature process.
   -us            unsigned source package.
@@ -108,23 +122,41 @@ sub usage {
 '), $Dpkg::PROGNAME;
 }
 
+my $admindir;
 my @debian_rules = ('debian/rules');
 my @rootcommand = ();
 my $signcommand;
-my ($admindir, $signkey, $usepause, $noclean,
-    $cleansource, $since, $maint,
-    $changedby, $desc, $parallel);
+my $noclean;
+my $cleansource;
+my $parallel;
 my $checkbuilddep = 1;
+my @checkbuilddep_opts;
+my @source_opts;
+my $check_command = $ENV{DEB_CHECK_COMMAND};
+my @check_opts;
+my $signpause;
+my $signkey = $ENV{DEB_SIGN_KEYID};
 my $signforce = 0;
 my $signreleased = 1;
 my $signsource = 1;
 my $signchanges = 1;
 my $buildtarget = 'build';
 my $binarytarget = 'binary';
-my $targetarch = my $targetgnusystem = '';
+my $targetarch = '';
+my $targetgnusystem = '';
+my @build_profiles = ();
 my $call_target = '';
 my $call_target_as_root = 0;
-my (@checkbuilddep_opts, @changes_opts, @source_opts);
+my $since;
+my $maint;
+my $changedby;
+my $desc;
+my @changes_opts;
+my @hook_names = qw(
+    init preclean source build binary changes postclean check sign done
+);
+my %hook;
+$hook{$_} = undef foreach @hook_names;
 
 use constant BUILD_DEFAULT    => 1;
 use constant BUILD_SOURCE     => 2;
@@ -138,18 +170,30 @@ sub build_normal() { return ($include & BUILD_ALL) == BUILD_ALL; }
 sub build_sourceonly() { return $include == BUILD_SOURCE; }
 sub build_binaryonly() { return !($include & BUILD_SOURCE); }
 sub build_binaryindep() { return ($include == BUILD_ARCH_INDEP); }
-sub build_opt() {
-    return (($include == BUILD_BINARY) ? '-b' :
-            (($include == BUILD_ARCH_DEP) ? '-B' :
-             (($include == BUILD_ARCH_INDEP) ? '-A' :
-              (($include == BUILD_SOURCE) ? '-S' :
-               internerr("build_opt called with include=$include")))));
+sub build_opt {
+    if ($include == BUILD_BINARY) {
+        return '-b';
+    } elsif ($include == BUILD_ARCH_DEP) {
+        return '-B';
+    } elsif ($include == BUILD_ARCH_INDEP) {
+        return '-A';
+    } elsif ($include == BUILD_SOURCE) {
+        return '-S';
+    } else {
+        croak "build_opt called with include=$include";
+    }
+}
+
+my $build_opts = Dpkg::BuildOptions->new();
+
+if ($build_opts->has('nocheck')) {
+    $check_command = undef;
 }
 
 while (@ARGV) {
     $_ = shift @ARGV;
 
-    if (/^(--help|-\?)$/) {
+    if (/^(?:--help|-\?)$/) {
 	usage;
 	exit 0;
     } elsif (/^--version$/) {
@@ -163,10 +207,22 @@ while (@ARGV) {
 	push @source_opts, $1;
     } elsif (/^--changes-option=(.*)$/) {
 	push @changes_opts, $1;
-    } elsif (/^-j(\d*)$/) {
+    } elsif (/^-j(\d*|auto)$/) {
 	$parallel = $1 || '';
     } elsif (/^-r(.*)$/) {
-	@rootcommand = split /\s+/, $1;
+	my $arg = $1;
+	@rootcommand = split /\s+/, $arg;
+    } elsif (/^--check-command=(.*)$/) {
+	$check_command = $1;
+    } elsif (/^--check-option=(.*)$/) {
+	push @check_opts, $1;
+    } elsif (/^--hook-(.+)=(.*)$/) {
+	my ($hook_name, $hook_cmd) = ($1, $2);
+	usageerr(_g('unknown hook name %s'), $hook_name)
+	    if not exists $hook{$hook_name};
+	usageerr(_g('missing hook %s command'), $hook_name)
+	    if not defined $hook_cmd;
+	$hook{$hook_name} = $hook_cmd;
     } elsif (/^-p(.*)$/) {
 	$signcommand = $1;
     } elsif (/^-k(.*)$/) {
@@ -183,9 +239,12 @@ while (@ARGV) {
     } elsif (/^-uc$/) {
 	$signchanges = 0;
     } elsif (/^-ap$/) {
-	$usepause = 1;
+	$signpause = 1;
     } elsif (/^-a(.*)$/) {
 	$targetarch = $1;
+    } elsif (/^-P(.*)$/) {
+	my $arg = $1;
+	@build_profiles = split /,/, $arg;
     } elsif (/^-s[iad]$/) {
 	push @changes_opts, $_;
     } elsif (/^-(?:s[insAkurKUR]|[zZ].*|i.*|I.*)$/) {
@@ -194,42 +253,47 @@ while (@ARGV) {
 	$cleansource = 1;
     } elsif (/^-t(.*)$/) {
 	$targetgnusystem = $1; # Order DOES matter!
-    } elsif (/^(--target|-T)$/) {
+    } elsif (/^(?:--target|-T)$/) {
         $call_target = shift @ARGV;
-    } elsif (/^(--target=|-T)(.+)$/) {
-        $call_target = $2;
+    } elsif (/^(?:--target=|-T)(.+)$/) {
+        $call_target = $1;
     } elsif (/^--as-root$/) {
         $call_target_as_root = 1;
     } elsif (/^-nc$/) {
 	$noclean = 1;
     } elsif (/^-b$/) {
-	build_sourceonly && usageerr(_g('cannot combine %s and %s'), $_, '-S');
+	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
+	    if build_sourceonly;
 	$include = BUILD_BINARY;
 	push @changes_opts, '-b';
 	@checkbuilddep_opts = ();
 	$buildtarget = 'build';
 	$binarytarget = 'binary';
     } elsif (/^-B$/) {
-	build_sourceonly && usageerr(_g('cannot combine %s and %s'), $_, '-S');
+	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
+	    if build_sourceonly;
 	$include = BUILD_ARCH_DEP;
 	push @changes_opts, '-B';
 	@checkbuilddep_opts = qw(-B);
 	$buildtarget = 'build-arch';
 	$binarytarget = 'binary-arch';
     } elsif (/^-A$/) {
-	build_sourceonly && usageerr(_g('cannot combine %s and %s'), $_, '-S');
+	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
+	    if build_sourceonly;
 	$include = BUILD_ARCH_INDEP;
 	push @changes_opts, '-A';
 	@checkbuilddep_opts = qw(-A);
 	$buildtarget = 'build-indep';
 	$binarytarget = 'binary-indep';
     } elsif (/^-S$/) {
-	build_binaryonly && usageerr(_g('cannot combine %s and %s'), build_opt, '-S');
+	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
+	    if build_binaryonly;
 	$include = BUILD_SOURCE;
 	push @changes_opts, '-S';
 	@checkbuilddep_opts = qw(-A -B);
     } elsif (/^-F$/) {
-	!build_normal && usageerr(_g('cannot combine %s and %s'), $_, build_opt);
+	usageerr(_g('cannot combine %s and %s'), build_opt(), $_)
+	    if not build_normal;
 	$include = BUILD_ALL;
 	@checkbuilddep_opts = ();
     } elsif (/^-v(.*)$/) {
@@ -244,7 +308,8 @@ while (@ARGV) {
 	# Deprecated option
 	warning(_g('-E and -W are deprecated, they are without effect'));
     } elsif (/^-R(.*)$/) {
-	@debian_rules = split /\s+/, $1;
+	my $arg = $1;
+	@debian_rules = split /\s+/, $arg;
     } else {
 	usageerr(_g('unknown option or argument %s'), $_);
     }
@@ -259,20 +324,43 @@ if ($< == 0) {
     warning(_g('using a gain-root-command while being root')) if (@rootcommand);
 } else {
     push @rootcommand, 'fakeroot' unless @rootcommand;
+}
 
-    if (!find_command($rootcommand[0])) {
-	if ($rootcommand[0] eq 'fakeroot') {
-	    error(_g("fakeroot not found, either install the fakeroot\n" .
-	             'package, specify a command with the -r option, ' .
-	             'or run this as root'));
-	} else {
-	    error(_g("gain-root-commmand '%s' not found"), $rootcommand[0]);
-	}
+if (@rootcommand and not find_command($rootcommand[0])) {
+    if ($rootcommand[0] eq 'fakeroot' and $< != 0) {
+        error(_g("fakeroot not found, either install the fakeroot\n" .
+                 'package, specify a command with the -r option, ' .
+                 'or run this as root'));
+    } else {
+        error(_g("gain-root-commmand '%s' not found"), $rootcommand[0]);
+    }
+}
+
+if ($check_command and not find_command($check_command)) {
+    error(_g("check-commmand '%s' not found"), $check_command);
+}
+
+if ($signcommand) {
+    if (!find_command($signcommand)) {
+        error(_g("sign-commmand '%s' not found"), $signcommand);
+    }
+} elsif (($ENV{GNUPGHOME} && -e $ENV{GNUPGHOME}) ||
+         ($ENV{HOME} && -e "$ENV{HOME}/.gnupg")) {
+    if (find_command('gpg2')) {
+        $signcommand = 'gpg2';
+    } elsif (find_command('gpg')) {
+        $signcommand = 'gpg';
     }
 }
 
-my $build_opts = Dpkg::BuildOptions->new();
 if (defined $parallel) {
+    if ($parallel eq 'auto') {
+        # Most Unices.
+        $parallel = qx(getconf _NPROCESSORS_ONLN 2>/dev/null);
+        # Fallback for at least Irix.
+        $parallel = qx(getconf _NPROC_ONLN 2>/dev/null) if $?;
+        chomp $parallel;
+    }
     $parallel = $build_opts->get('parallel') if $build_opts->has('parallel');
     $ENV{MAKEFLAGS} ||= '';
     $ENV{MAKEFLAGS} .= " -j$parallel";
@@ -280,6 +368,8 @@ if (defined $parallel) {
     $build_opts->export();
 }
 
+set_build_profiles(@build_profiles) if @build_profiles;
+
 my $cwd = cwd();
 my $dir = basename($cwd);
 
@@ -287,9 +377,13 @@ my $changelog = changelog_parse();
 
 my $pkg = mustsetvar($changelog->{source}, _g('source package'));
 my $version = mustsetvar($changelog->{version}, _g('source version'));
-my ($ok, $error) = version_check($version);
+my $v = Dpkg::Version->new($version);
+my ($ok, $error) = version_check($v);
 error($error) unless $ok;
 
+my $sversion = $v->as_string(omit_epoch => 1);
+my $uversion = $v->version();
+
 my $distribution = mustsetvar($changelog->{distribution}, _g('source distribution'));
 
 my $maintainer;
@@ -319,12 +413,8 @@ if (build_sourceonly) {
     $arch = mustsetvar($ENV{DEB_HOST_ARCH}, _g('host architecture'));
 }
 
-if (!defined $signcommand &&
-    (($ENV{GNUPGHOME} && -e $ENV{GNUPGHOME}) ||
-     ($ENV{HOME} && -e "$ENV{HOME}/.gnupg")) &&
-    find_command('gpg')) {
-    $signcommand = 'gpg';
-}
+my $pv = "${pkg}_$sversion";
+my $pva = "${pkg}_${sversion}_$arch";
 
 if (not $signcommand) {
     $signsource = 0;
@@ -338,12 +428,15 @@ if (not $signcommand) {
     $signchanges = 0;
 }
 
+if ($signsource && build_binaryonly) {
+    $signsource = 0;
+}
+
+#
 # Preparation of environment stops here
+#
 
-(my $sversion = $version) =~ s/^\d+://;
-
-my $pv = "${pkg}_$sversion";
-my $pva = "${pkg}_${sversion}_$arch";
+run_hook('init', 1);
 
 if (not -x 'debian/rules') {
     warning(_g('debian/rules is not executable; fixing that'));
@@ -357,9 +450,7 @@ unless ($call_target) {
 }
 
 if ($checkbuilddep) {
-    if ($admindir) {
-	push @checkbuilddep_opts, "--admindir=$admindir";
-    }
+    push @checkbuilddep_opts, "--admindir=$admindir" if $admindir;
 
     system('dpkg-checkbuilddeps', @checkbuilddep_opts);
     if (not WIFEXITED($?)) {
@@ -388,10 +479,15 @@ if ($call_target) {
     exit 0;
 }
 
+run_hook('preclean', ! $noclean);
+
 unless ($noclean) {
     withecho(@rootcommand, @debian_rules, 'clean');
 }
-unless (build_binaryonly) {
+
+run_hook('source', $include & BUILD_SOURCE);
+
+if ($include & BUILD_SOURCE) {
     warning(_g('building a source package without cleaning up as you asked; ' .
                'it might contain undesired files')) if $noclean;
     chdir('..') or syserr('chdir ..');
@@ -399,6 +495,8 @@ unless (build_binaryonly) {
     chdir($dir) or syserr("chdir $dir");
 }
 
+run_hook('build', $include & BUILD_BINARY);
+
 if ($buildtarget ne 'build' and scalar(@debian_rules) == 1) {
     # Verify that build-{arch,indep} are supported. If not, fallback to build.
     # This is a temporary measure to not break too many packages on a flag day.
@@ -417,97 +515,84 @@ if ($buildtarget ne 'build' and scalar(@debian_rules) == 1) {
     }
 }
 
-unless (build_sourceonly) {
+if ($include & BUILD_BINARY) {
     withecho(@debian_rules, $buildtarget);
+    run_hook('binary', 1);
     withecho(@rootcommand, @debian_rules, $binarytarget);
 }
-if ($usepause &&
-    ($signchanges || (!build_binaryonly && $signsource))) {
-    print _g("Press the return key to start signing process\n");
-    getc();
-}
 
-my $signerrors;
-unless (build_binaryonly) {
-    if ($signsource && signfile("$pv.dsc")) {
-	$signerrors = _g('Failed to sign .dsc and .changes file');
-	$signchanges = 0;
-    }
-}
+run_hook('changes', 1);
 
-if (defined($maint)) { push @changes_opts, "-m$maint" }
-if (defined($changedby)) { push @changes_opts, "-e$changedby" }
-if (defined($since)) { push @changes_opts, "-v$since" }
-if (defined($desc)) { push @changes_opts, "-C$desc" }
+push @changes_opts, "-m$maint" if defined $maint;
+push @changes_opts, "-e$changedby" if defined $changedby;
+push @changes_opts, "-v$since" if defined $since;
+push @changes_opts, "-C$desc" if defined $desc;
 
 my $chg = "../$pva.changes";
-print STDERR " dpkg-genchanges @changes_opts >$chg\n";
+my $changes = Dpkg::Control->new(type => CTRL_FILE_CHANGES);
+
+print { *STDERR } " dpkg-genchanges @changes_opts >$chg\n";
+
 open my $changes_fh, '-|', 'dpkg-genchanges', @changes_opts
     or subprocerr('dpkg-genchanges');
-
-open my $out_fh, '>', $chg or syserr(_g('write changes file'));
-
-my $infiles = my $files = '';
-while ($_ = <$changes_fh>) {
-    print $out_fh $_ or syserr(_g('write changes file'));
-    chomp;
-
-    if (/^Files:/i) {
-	$infiles = 1;
-    } elsif ($infiles && /^\s+(.*)$/) {
-	$files .= " $1 ";
-    } elsif ($infiles && /^\S/) {
-	$infiles = 0;
-    }
-}
-
+$changes->parse($changes_fh, _g('parse changes file'));
+$changes->save($chg);
 close $changes_fh or subprocerr(_g('dpkg-genchanges'));
-close $out_fh or syserr(_g('write changes file'));
 
-my $srcmsg;
-sub fileomitted($) { return $files !~ /$_[0]/ }
-my $ext = $compression_re_file_ext;
-if (fileomitted '\.deb') {
-    # source only upload
-    if (fileomitted "\.diff\.$ext" and fileomitted "\.debian\.tar\.$ext") {
-	$srcmsg = _g('source only upload: Debian-native package');
-    } elsif (fileomitted "\.orig\.tar\.$ext") {
-	$srcmsg = _g('source only, diff-only upload (original source NOT included)');
-    } else {
-	$srcmsg = _g('source only upload (original source is included)');
-    }
-} else {
-    $srcmsg = _g('full upload (original source is included)');
-    if (fileomitted '\.dsc') {
-	$srcmsg = _g('binary only upload (no source included)');
-    } elsif (fileomitted "\.diff\.$ext" and fileomitted "\.debian\.tar\.$ext") {
-	$srcmsg = _g('full upload; Debian-native package (full source is included)');
-    } elsif (fileomitted "\.orig\.tar\.$ext") {
-	$srcmsg = _g('binary and diff upload (original source NOT included)');
-    } else {
-	$srcmsg = _g('full upload (original source is included)');
-    }
-}
-
-if ($signchanges && signfile("$pva.changes")) {
-    $signerrors = _g('Failed to sign .changes file');
-}
+run_hook('postclean', $cleansource);
 
 if ($cleansource) {
     withecho(@rootcommand, @debian_rules, 'clean');
 }
+
 chdir('..') or syserr('chdir ..');
 withecho('dpkg-source', @source_opts, '--after-build', $dir);
 chdir($dir) or syserr("chdir $dir");
 
+printf "$Dpkg::PROGNAME: %s\n", describe_build($changes->{'Files'});
+
+run_hook('check', $check_command);
+
+if ($check_command) {
+    withecho($check_command, @check_opts, $chg);
+}
+
+if ($signpause && ($signchanges || $signsource)) {
+    print _g("Press the return key to start signing process\n");
+    getc();
+}
+
+run_hook('sign', $signsource || $signchanges);
+
+if ($signsource) {
+    if (signfile("$pv.dsc")) {
+        error(_g('failed to sign .dsc and .changes file'));
+    }
+
+    # Recompute the checksums as the .dsc has changed now.
+    my $checksums = Dpkg::Checksums->new();
+    $checksums->add_from_control($changes);
+    $checksums->add_from_file("../$pv.dsc", update => 1, key => "$pv.dsc");
+    $checksums->export_to_control($changes);
+    delete $changes->{'Checksums-Md5'};
+
+    my $md5sum_regex = checksums_get_property('md5', 'regex');
+    my $dsc_md5sum = $checksums->get_checksum("$pv.dsc", 'md5');
+    my $dsc_size = $checksums->get_size("$pv.dsc");
+    my $dsc_files_regex = qr/$md5sum_regex\s+\d+\s+(\S+\s+\S+\s+\Q$pv\E\.dsc)/;
+    $changes->{'Files'} =~ s/^$dsc_files_regex$/$dsc_md5sum $dsc_size $1/m;
+
+    $changes->save($chg);
+}
+if ($signchanges && signfile("$pva.changes")) {
+    error(_g('failed to sign .changes file'));
+}
+
 if (not $signreleased) {
     warning(_g('not signing UNRELEASED build; use --force-sign to override'));
 }
-print "$Dpkg::PROGNAME: $srcmsg\n";
-if ($signerrors) {
-    warning($signerrors);
-    exit 1;
-}
+
+run_hook('done', 1);
 
 sub mustsetvar {
     my ($var, $text) = @_;
@@ -520,15 +605,47 @@ sub mustsetvar {
 }
 
 sub withecho {
-    shift while !$_[0];
-    print STDERR " @_\n";
+    print { *STDERR } " @_\n";
     system(@_)
 	and subprocerr("@_");
 }
 
+sub run_hook {
+    my ($name, $enabled) = @_;
+    my $cmd = $hook{$name};
+
+    return if not $cmd;
+
+    print { *STDERR } "$Dpkg::PROGNAME: running hook $name\n";
+
+    my %hook_vars = (
+        '%' => '%',
+        'a' => $enabled ? 1 : 0,
+        'p' => $pkg,
+        'v' => $version,
+        's' => $sversion,
+        'u' => $uversion,
+    );
+
+    my $subst_hook_var = sub {
+        my ($var) = @_;
+
+        if (exists $hook_vars{$var}) {
+            return $hook_vars{$var};
+        } else {
+            warning(_g('unknown %% substitution in hook: %%%s'), $var);
+            return "\%$var";
+        }
+    };
+
+    $cmd =~ s/\%(.)/&$subst_hook_var($1)/eg;
+
+    withecho($cmd);
+}
+
 sub signfile {
     my ($file) = @_;
-    print STDERR " signfile $file\n";
+    print { *STDERR } " signfile $file\n";
     my $qfile = quotemeta($file);
 
     system("(cat ../$qfile ; echo '') | " .
@@ -546,3 +663,35 @@ sub signfile {
     print "\n";
     return $status
 }
+
+sub fileomitted {
+    my ($files, $regex) = @_;
+
+    return $files !~ /$regex/
+}
+
+sub describe_build {
+    my ($files) = @_;
+    my $ext = compression_get_file_extension_regex();
+
+    if (fileomitted($files, qr/\.deb/)) {
+        # source-only upload
+        if (fileomitted($files, qr/\.diff\.$ext/) and
+            fileomitted($files, qr/\.debian\.tar\.$ext/)) {
+            return _g('source-only upload: Debian-native package');
+        } elsif (fileomitted($files, qr/\.orig\.tar\.$ext/)) {
+            return _g('source-only, diff-only upload (original source NOT included)');
+        } else {
+            return _g('source-only upload (original source is included)');
+        }
+    } elsif (fileomitted($files, qr/\.dsc/)) {
+        return _g('binary-only upload (no source included)');
+    } elsif (fileomitted($files, qr/\.diff\.$ext/) and
+             fileomitted($files, qr/\.debian\.tar\.$ext/)) {
+        return _g('full upload; Debian-native package (full source is included)');
+    } elsif (fileomitted($files, qr/\.orig\.tar\.$ext/)) {
+        return _g('binary and diff upload (original source NOT included)');
+    } else {
+        return _g('full upload (original source is included)');
+    }
+}
