@@ -4,7 +4,7 @@
 #
 # Copyright © 1996 Ian Jackson
 # Copyright © 2000,2002 Wichert Akkerman
-# Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -30,6 +30,7 @@ use Dpkg::Util qw(:list);
 use Dpkg::File;
 use Dpkg::Arch qw(get_host_arch debarch_eq debarch_is);
 use Dpkg::Package;
+use Dpkg::BuildProfiles qw(get_build_profiles);
 use Dpkg::Deps;
 use Dpkg::Control;
 use Dpkg::Control::Info;
@@ -37,6 +38,7 @@ use Dpkg::Control::Fields;
 use Dpkg::Substvars;
 use Dpkg::Vars;
 use Dpkg::Changelog::Parse;
+use Dpkg::Dist::Files;
 
 textdomain('dpkg-dev');
 
@@ -46,6 +48,7 @@ my $changelogfile = 'debian/changelog';
 my $changelogformat;
 my $fileslistfile = 'debian/files';
 my $packagebuilddir = 'debian/tmp';
+my $outputfile;
 
 my $sourceversion;
 my $binaryversion;
@@ -81,7 +84,7 @@ sub usage {
   -f<files-list-file>      write files here instead of debian/files.
   -P<package-build-dir>    temporary build dir instead of debian/tmp.
   -n<filename>             assume the package filename will be <filename>.
-  -O                       write to stdout, not .../DEBIAN/control.
+  -O[<file>]               write to stdout (or <file>), not .../DEBIAN/control.
   -is, -ip, -isp, -ips     deprecated, ignored for compatibility.
   -D<field>=<value>        override or add a field and value.
   -U<field>                remove a field.
@@ -94,38 +97,40 @@ sub usage {
 
 while (@ARGV) {
     $_=shift(@ARGV);
-    if (m/^-p/) {
-        $oppackage = $';
+    if (m/^-p/p) {
+        $oppackage = ${^POSTMATCH};
         my $err = pkg_name_is_illegal($oppackage);
         error(_g("illegal package name '%s': %s"), $oppackage, $err) if $err;
-    } elsif (m/^-c/) {
-        $controlfile= $';
-    } elsif (m/^-l/) {
-        $changelogfile= $';
-    } elsif (m/^-P/) {
-        $packagebuilddir= $';
-    } elsif (m/^-f/) {
-        $fileslistfile= $';
+    } elsif (m/^-c/p) {
+        $controlfile = ${^POSTMATCH};
+    } elsif (m/^-l/p) {
+        $changelogfile = ${^POSTMATCH};
+    } elsif (m/^-P/p) {
+        $packagebuilddir = ${^POSTMATCH};
+    } elsif (m/^-f/p) {
+        $fileslistfile = ${^POSTMATCH};
     } elsif (m/^-v(.+)$/) {
         $forceversion= $1;
     } elsif (m/^-O$/) {
         $stdout= 1;
+    } elsif (m/^-O(.+)$/) {
+        $outputfile = $1;
     } elsif (m/^-i[sp][sp]?$/) {
 	# ignored for backwards compatibility
     } elsif (m/^-F([0-9a-z]+)$/) {
         $changelogformat=$1;
-    } elsif (m/^-D([^\=:]+)[=:]/) {
-        $override{$1}= $';
+    } elsif (m/^-D([^\=:]+)[=:]/p) {
+        $override{$1} = ${^POSTMATCH};
     } elsif (m/^-U([^\=:]+)$/) {
         $remove{$1}= 1;
-    } elsif (m/^-V(\w[-:0-9A-Za-z]*)[=:]/) {
-        $substvars->set_as_used($1, $');
+    } elsif (m/^-V(\w[-:0-9A-Za-z]*)[=:]/p) {
+        $substvars->set_as_used($1, ${^POSTMATCH});
     } elsif (m/^-T(.*)$/) {
 	$substvars->load($1) if -e $1;
 	$substvars_loaded = 1;
-    } elsif (m/^-n/) {
-        $forcefilename= $';
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-n/p) {
+        $forcefilename = ${^POSTMATCH};
+    } elsif (m/^-(?:\?|-help)$/) {
         usage();
         exit(0);
     } elsif (m/^--version$/) {
@@ -169,7 +174,9 @@ my $pkg;
 
 if (defined($oppackage)) {
     $pkg = $control->get_pkg_by_name($oppackage);
-    defined($pkg) || error(_g('package %s not in control info'), $oppackage);
+    if (not defined $pkg) {
+        error(_g('package %s not in control info'), $oppackage)
+    }
 } else {
     my @packages = map { $_->{'Package'} } $control->get_packages();
     if (@packages == 0) {
@@ -205,9 +212,9 @@ foreach (keys %{$pkg}) {
 	} else {
 	    my @archlist = split(/\s+/, $v);
 	    my @invalid_archs = grep { m/[^\w-]/ } @archlist;
-	    warning(ngettext("`%s' is not a legal architecture string.",
-			     "`%s' are not legal architecture strings.",
-			     scalar(@invalid_archs)),
+	    warning(P_("`%s' is not a legal architecture string.",
+	               "`%s' are not legal architecture strings.",
+	               scalar(@invalid_archs)),
 		    join("' `", @invalid_archs))
 		if @invalid_archs >= 1;
 	    if (none { debarch_is($host_arch, $_) } @archlist) {
@@ -248,7 +255,7 @@ $facts->add_installed_package($fields->{'Package'}, $fields->{'Version'},
                               $fields->{'Architecture'}, $fields->{'Multi-Arch'});
 if (exists $pkg->{'Provides'}) {
     my $provides = deps_parse($substvars->substvars($pkg->{'Provides'}, no_warn => 1),
-                              reduce_arch => 1, union => 1);
+                              reduce_restrictions => 1, union => 1);
     if (defined $provides) {
 	foreach my $subdep ($provides->get_deps()) {
 	    if ($subdep->isa('Dpkg::Deps::Simple')) {
@@ -270,7 +277,8 @@ foreach my $field (field_list_pkg_dep()) {
 	    msg_prefix => sprintf(_g('%s field of package %s: '), $field, $pkg->{Package}));
 	if (field_get_dep_type($field) eq 'normal') {
 	    $dep = deps_parse($field_value, use_arch => 1,
-			      reduce_arch => $reduce_arch);
+	                      reduce_arch => $reduce_arch,
+	                      reduce_profiles => 1);
 	    error(_g('error occurred while parsing %s field: %s'), $field,
                   $field_value) unless defined $dep;
 	    $dep->simplify_deps($facts, @seen_deps);
@@ -278,7 +286,8 @@ foreach my $field (field_list_pkg_dep()) {
 	    push @seen_deps, $dep;
 	} else {
 	    $dep = deps_parse($field_value, use_arch => 1,
-                              reduce_arch => $reduce_arch, union => 1);
+	                      reduce_arch => $reduce_arch,
+	                      reduce_profiles => 1, union => 1);
 	    error(_g('error occurred while parsing %s field: %s'), $field,
                   $field_value) unless defined $dep;
 	    $dep->simplify_deps($facts);
@@ -292,11 +301,15 @@ foreach my $field (field_list_pkg_dep()) {
     }
 }
 
+$fields->{'Built-For-Profiles'} = join ' ', get_build_profiles();
+
 for my $f (qw(Package Version)) {
-    defined($fields->{$f}) || error(_g('missing information for output field %s'), $f);
+    error(_g('missing information for output field %s'), $f)
+        unless defined $fields->{$f};
 }
 for my $f (qw(Maintainer Description Architecture)) {
-    defined($fields->{$f}) || warning(_g('missing information for output field %s'), $f);
+    warning(_g('missing information for output field %s'), $f)
+        unless defined $fields->{$f};
 }
 $oppackage = $fields->{'Package'};
 
@@ -313,6 +326,7 @@ if ($pkg_type eq 'udeb') {
     }
 }
 
+my $sourcepackage = get_source_package();
 my $verdiff = $binaryversion ne $sourceversion;
 if ($oppackage ne $sourcepackage || $verdiff) {
     $fields->{'Source'} = $sourcepackage;
@@ -320,22 +334,25 @@ if ($oppackage ne $sourcepackage || $verdiff) {
 }
 
 if (!defined($substvars->get('Installed-Size'))) {
-    my $du_fh;
-    defined(my $c = open($du_fh, '-|')) || syserr(_g('cannot fork for %s'), 'du');
+    my $c = open(my $du_fh, '-|');
+    if (not defined $c) {
+        syserr(_g('cannot fork for %s'), 'du');
+    }
     if (!$c) {
-        chdir("$packagebuilddir") ||
-            syserr(_g("chdir for du to \`%s'"), $packagebuilddir);
-        exec('du', '-k', '-s', '--apparent-size', '.') or
-            syserr(_g('unable to execute %s'), 'du');
+        chdir("$packagebuilddir")
+            or syserr(_g("chdir for du to \`%s'"), $packagebuilddir);
+        exec('du', '-k', '-s', '--apparent-size', '.')
+            or syserr(_g('unable to execute %s'), 'du');
     }
     my $duo = '';
     while (<$du_fh>) {
 	$duo .= $_;
     }
     close($du_fh);
-    $? && subprocerr(_g("du in \`%s'"), $packagebuilddir);
-    $duo =~ m/^(\d+)\s+\.$/ ||
+    subprocerr(_g("du in \`%s'"), $packagebuilddir) if $?;
+    if ($duo !~ m/^(\d+)\s+\.$/) {
         error(_g("du gave unexpected output \`%s'"), $duo);
+    }
     $substvars->set_as_used('Installed-Size', $1);
 }
 if (defined($substvars->get('Extra-Size'))) {
@@ -353,56 +370,52 @@ for my $f (keys %remove) {
     delete $fields->{$f};
 }
 
-# Obtain a lock on debian/control to avoid simultaneous updates
-# of debian/files when parallel building is in use
-my $lockfh;
-sysopen($lockfh, 'debian/control', O_WRONLY) ||
-    syserr(_g('cannot write %s'), 'debian/control');
-file_lock($lockfh, 'debian/control');
-
-$fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
-open(my $fileslistnew_fh, '>', "$fileslistfile.new") ||
-    syserr(_g('open new files list file'));
-binmode($fileslistnew_fh);
-if (open(my $fileslist_fh, '<', $fileslistfile)) {
-    binmode($fileslist_fh);
-    while (<$fileslist_fh>) {
-        chomp;
-        next if m/^([-+0-9a-z.]+)_[^_]+_([\w-]+)\.(a-z+) /
-                && ($1 eq $oppackage)
-	        && ($3 eq $pkg_type)
-	        && (debarch_eq($2, $fields->{'Architecture'} || '')
-		    || debarch_eq($2, 'all'));
-        print($fileslistnew_fh "$_\n") ||
-            syserr(_g('copy old entry to new files list file'));
-    }
-    close($fileslist_fh) || syserr(_g('close old files list file'));
-} elsif ($! != ENOENT) {
-    syserr(_g('read old files list file'));
-}
 my $sversion = $fields->{'Version'};
 $sversion =~ s/^\d+://;
 $forcefilename //= sprintf('%s_%s_%s.%s', $oppackage, $sversion,
                            $fields->{'Architecture'} || '', $pkg_type);
+$forcefilename = $substvars->substvars($forcefilename);
+my $section = $substvars->substvars($fields->{'Section'} || '-');
+my $priority = $substvars->substvars($fields->{'Priority'} || '-');
 
-print($fileslistnew_fh $substvars->substvars(sprintf("%s %s %s\n",
-                                             $forcefilename,
-                                             $fields->{'Section'} || '-',
-                                             $fields->{'Priority'} || '-')))
-    || syserr(_g('write new entry to new files list file'));
-close($fileslistnew_fh) || syserr(_g('close new files list file'));
-rename("$fileslistfile.new", $fileslistfile) || syserr(_g('install new files list file'));
+# Obtain a lock on debian/control to avoid simultaneous updates
+# of debian/files when parallel building is in use
+my $lockfh;
+my $lockfile = 'debian/control';
+$lockfile = $controlfile if not -e $lockfile;
+
+sysopen($lockfh, $lockfile, O_WRONLY)
+    or syserr(_g('cannot write %s'), $lockfile);
+file_lock($lockfh, $lockfile);
+
+my $dist = Dpkg::Dist::Files->new();
+$dist->load($fileslistfile) if -e $fileslistfile;
+
+foreach my $file ($dist->get_files()) {
+    if (defined $file->{package} &&
+        ($file->{package} eq $oppackage) &&
+        ($file->{package_type} eq $pkg_type) &&
+        (debarch_eq($file->{arch}, $fields->{'Architecture'} || '') ||
+         debarch_eq($file->{arch}, 'all'))) {
+        $dist->del_file($file->{filename});
+    }
+}
+
+$dist->add_file($forcefilename, $section, $priority);
+$dist->save("$fileslistfile.new");
+
+rename("$fileslistfile.new", $fileslistfile)
+    or syserr(_g('install new files list file'));
 
 # Release the lock
-close($lockfh) || syserr(_g('cannot close %s'), 'debian/control');
+close($lockfh) or syserr(_g('cannot close %s'), $lockfile);
 
 my $cf;
 my $fh_output;
 if (!$stdout) {
-    $cf= "$packagebuilddir/DEBIAN/control";
-    $cf= "./$cf" if $cf =~ m/^\s/;
-    open($fh_output, '>', "$cf.new") ||
-        syserr(_g("cannot open new output control file \`%s'"), "$cf.new");
+    $cf = $outputfile // "$packagebuilddir/DEBIAN/control";
+    open($fh_output, '>', "$cf.new")
+        or syserr(_g("cannot open new output control file \`%s'"), "$cf.new");
 } else {
     $fh_output = \*STDOUT;
 }
@@ -411,9 +424,9 @@ $fields->apply_substvars($substvars);
 $fields->output($fh_output);
 
 if (!$stdout) {
-    close($fh_output) || syserr(_g('cannot close %s'), "$cf.new");
-    rename("$cf.new", "$cf") ||
-        syserr(_g("cannot install output control file \`%s'"), $cf);
+    close($fh_output) or syserr(_g('cannot close %s'), "$cf.new");
+    rename("$cf.new", "$cf")
+        or syserr(_g("cannot install output control file \`%s'"), $cf);
 }
 
 $substvars->warn_about_unused();
