@@ -3,7 +3,7 @@
 # dpkg-gensymbols
 #
 # Copyright © 2007 Raphaël Hertzog
-# Copyright © 2007-2012 Guillem Jover <guillem@debian.org>
+# Copyright © 2007-2013 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -24,7 +24,7 @@ use warnings;
 use Dpkg ();
 use Dpkg::Arch qw(get_host_arch);
 use Dpkg::Package;
-use Dpkg::Shlibs qw(@librarypaths);
+use Dpkg::Shlibs qw(get_library_paths);
 use Dpkg::Shlibs::Objdump;
 use Dpkg::Shlibs::SymbolFile;
 use Dpkg::Gettext;
@@ -77,8 +77,7 @@ sub usage {
                            file and the reference template.
   -I<file>                 force usage of <file> as reference symbols
                            file instead of the default file.
-  -O<file>                 write to <file>, not .../DEBIAN/symbols.
-  -O                       write to stdout, not .../DEBIAN/symbols.
+  -O[<file>]               write to stdout (or <file>), not .../DEBIAN/symbols.
   -t                       write in template mode (tags are not
                            processed and included in output).
   -V                       verbose output; write deprecated symbols and pattern
@@ -94,12 +93,12 @@ sub usage {
 my @files;
 while (@ARGV) {
     $_ = shift(@ARGV);
-    if (m/^-p/) {
-	$oppackage = $';
+    if (m/^-p/p) {
+	$oppackage = ${^POSTMATCH};
 	my $err = pkg_name_is_illegal($oppackage);
 	error(_g("illegal package name '%s': %s"), $oppackage, $err) if $err;
     } elsif (m/^-c(\d)?$/) {
-	$compare = defined($1) ? $1 : 1;
+	$compare = $1 // 1;
     } elsif (m/^-q$/) {
 	$quiet = 1;
     } elsif (m/^-d$/) {
@@ -131,7 +130,7 @@ while (@ARGV) {
 	$verbose_output = 1;
     } elsif (m/^-a(.+)$/) {
 	$host_arch = $1;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-(?:\?|-help)$/) {
 	usage();
 	exit(0);
     } elsif (m/^--version$/) {
@@ -181,7 +180,7 @@ foreach my $file ($input, $output, "debian/$oppackage.symbols.$host_arch",
 
 # Scan package build dir looking for libraries
 if (not scalar @files) {
-    PATH: foreach my $path (@librarypaths) {
+    PATH: foreach my $path (get_library_paths()) {
 	my $libdir = "$packagebuilddir$path";
 	$libdir =~ s{/+}{/}g;
 	lstat $libdir;
@@ -193,8 +192,8 @@ if (not scalar @files) {
                not check_files_are_the_same($packagebuilddir, $updir)) {
             next PATH if -l $updir;
         }
-	opendir(my $libdir_dh, "$libdir") ||
-	    syserr(_g("Can't read directory %s: %s"), $libdir, $!);
+	opendir(my $libdir_dh, "$libdir")
+	    or syserr(_g("can't read directory %s: %s"), $libdir, $!);
 	push @files, grep {
 	    /(\.so\.|\.so$)/ && -f $_ &&
 	    Dpkg::Shlibs::Objdump::is_elf($_);
@@ -209,7 +208,7 @@ foreach my $file (@files) {
     print "Scanning $file for symbol information\n" if $debug;
     my $objid = $od->analyze($file);
     unless (defined($objid) && $objid) {
-	warning(_g("Objdump couldn't parse %s\n"), $file);
+	warning(_g("Dpkg::Shlibs::Objdump couldn't parse %s\n"), $file);
 	next;
     }
     my $object = $od->get_object($objid);
@@ -234,7 +233,7 @@ if ($stdout) {
                      with_deprecated => $verbose_output);
 } else {
     unless (defined($output)) {
-	unless($symfile->is_empty()) {
+	unless ($symfile->is_empty()) {
 	    $output = "$packagebuilddir/DEBIAN/symbols";
 	    mkdir("$packagebuilddir/DEBIAN") if not -e "$packagebuilddir/DEBIAN";
 	}
