@@ -5,7 +5,7 @@
 # Copyright © 1996 Ian Jackson
 # Copyright © 2000 Wichert Akkerman
 # Copyright © 2006 Frank Lichtenheld
-# Copyright © 2006-2010,2012 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2010,2012-2013 Guillem Jover <guillem@debian.org>
 # Copyright © 2007 Raphaël Hertzog
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -35,7 +35,7 @@ use Dpkg::Util qw(:list);
 use Dpkg::Path qw(relative_to_pkg_root guess_pkg_root_dir
 		  check_files_are_the_same get_control_path);
 use Dpkg::Version;
-use Dpkg::Shlibs qw(find_library @librarypaths);
+use Dpkg::Shlibs qw(find_library get_library_paths);
 use Dpkg::Shlibs::Objdump;
 use Dpkg::Shlibs::SymbolFile;
 use Dpkg::Arch qw(get_host_arch);
@@ -93,19 +93,23 @@ foreach (@ARGV) {
 	push @pkg_dir_to_search, $1;
     } elsif (m/^-O$/) {
 	$stdout = 1;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-O(.+)$/) {
+	$varlistfile = $1;
+    } elsif (m/^-(?:\?|-help)$/) {
 	usage(); exit(0);
     } elsif (m/^--version$/) {
 	version(); exit(0);
     } elsif (m/^--admindir=(.*)$/) {
 	$admindir = $1;
-	-d $admindir ||
+	if (not -d $admindir) {
 	    error(_g("administrative directory '%s' does not exist"), $admindir);
+	}
 	$ENV{DPKG_ADMINDIR} = $admindir;
     } elsif (m/^-d(.*)$/) {
 	$dependencyfield = field_capitalize($1);
-	defined($depstrength{$dependencyfield}) ||
+	if (not defined $depstrength{$dependencyfield}) {
 	    warning(_g("unrecognized dependency field '%s'"), $dependencyfield);
+	}
     } elsif (m/^-e(.*)$/) {
 	if (exists $exec{$1}) {
 	    # Affect the binary to the most important field
@@ -138,13 +142,12 @@ foreach (@ARGV) {
 	}
     }
 }
-
-scalar keys %exec || usageerr(_g('need at least one executable'));
+usageerr(_g('need at least one executable')) unless scalar keys %exec;
 
 my $control = Dpkg::Control::Info->new();
 my $fields = $control->get_source();
 my $bd_value = deps_concat($fields->{'Build-Depends'}, $fields->{'Build-Depends-Arch'});
-my $build_deps = deps_parse($bd_value, build_dep => 1, reduce_arch => 1);
+my $build_deps = deps_parse($bd_value, build_dep => 1, reduce_restrictions => 1);
 error(_g('error occurred while parsing %s'), 'Build-Depends/Build-Depends-Arch')
     unless defined $build_deps;
 
@@ -320,7 +323,7 @@ foreach my $file (keys %exec) {
     my $in_public_dir = 1;
     if (my $relname = relative_to_pkg_root($file)) {
         my $parent_dir = '/' . dirname($relname);
-        $in_public_dir = any { $parent_dir eq $_ } @librarypaths;
+        $in_public_dir = any { $parent_dir eq $_ } get_library_paths();
     } else {
         warning(_g('binaries to analyze should already be ' .
                    "installed in their package's directory"));
@@ -454,16 +457,16 @@ my $fh;
 if ($stdout) {
     $fh = \*STDOUT;
 } else {
-    open(my $new_fh, '>', "$varlistfile.new") ||
-	syserr(_g("open new substvars file \`%s'"), "$varlistfile.new");
+    open(my $new_fh, '>', "$varlistfile.new")
+        or syserr(_g("open new substvars file \`%s'"), "$varlistfile.new");
     if (-e $varlistfile) {
-	open(my $old_fh, '<', $varlistfile) ||
-	    syserr(_g("open old varlist file \`%s' for reading"), $varlistfile);
+	open(my $old_fh, '<', $varlistfile)
+	    or syserr(_g("open old varlist file \`%s' for reading"), $varlistfile);
 	while (my $entry = <$old_fh>) {
 	    next if $entry =~ m/^\Q$varnameprefix\E:/;
-	    print($new_fh $entry) ||
-	        syserr(_g("copy old entry to new varlist file \`%s'"),
-	               "$varlistfile.new");
+	    print { $new_fh } $entry
+	        or syserr(_g("copy old entry to new varlist file \`%s'"),
+	                  "$varlistfile.new");
 	}
 	close($old_fh);
     }
@@ -528,15 +531,15 @@ foreach my $field (reverse @depfields) {
         my $obj = deps_parse($dep);
         error(_g('invalid dependency got generated: %s'), $dep) unless defined $obj;
         $obj->sort();
-	print $fh "$varnameprefix:$field=$obj\n";
+	print { $fh } "$varnameprefix:$field=$obj\n";
     }
 }
 
 # Replace old file by new one
 if (!$stdout) {
-    close($fh) || syserr(_g('cannot close %s'), "$varlistfile.new");
-    rename("$varlistfile.new",$varlistfile) ||
-	syserr(_g("install new varlist file \`%s'"), $varlistfile);
+    close($fh) or syserr(_g('cannot close %s'), "$varlistfile.new");
+    rename "$varlistfile.new", $varlistfile
+        or syserr(_g("install new varlist file \`%s'"), $varlistfile);
 }
 
 ##
@@ -564,7 +567,7 @@ sub usage {
 "Options:
   -l<library-dir>          add directory to private shared library search list.
   -p<varname-prefix>       set <varname-prefix>:* instead of shlibs:*.
-  -O                       print variable settings to stdout.
+  -O[<file>]               write variable settings to stdout (or <file>).
   -L<local-shlibs-file>    shlibs override file, not debian/shlibs.local.
   -T<substvars-file>       update variables here, not debian/substvars.
   -t<type>                 set package type (default is deb).
@@ -635,7 +638,7 @@ sub add_shlibs_dep {
     my ($soname, $pkg, $libfile) = @_;
     my @shlibs = ($shlibslocal, $shlibsoverride);
     if ($pkg eq '') {
-	# If the file is not packaged, try to find out the shlibs file in
+	# If the file is not packaged, try to find out the shlibs file in
 	# the package being built where the lib has been found
 	my $pkg_root = guess_pkg_root_dir($libfile);
 	if (defined $pkg_root) {
@@ -682,14 +685,13 @@ sub extract_from_shlibs {
     # Split soname in name/version
     my ($libname, $libversion) = split_soname($soname);
     unless (defined $libname) {
-	warning(_g("Can't extract name and version from library name \`%s'"),
+	warning(_g("can't extract name and version from library name '%s'"),
 	        $soname);
 	return;
     }
     # Open shlibs file
-    $shlibfile = "./$shlibfile" if $shlibfile =~ m/^\s/;
-    open(my $shlibs_fh, '<', $shlibfile) ||
-        syserr(_g("unable to open shared libs info file \`%s'"), $shlibfile);
+    open(my $shlibs_fh, '<', $shlibfile)
+        or syserr(_g("unable to open shared libs info file \`%s'"), $shlibfile);
     my $dep;
     while (<$shlibs_fh>) {
 	s/\s*\n$//;
@@ -699,7 +701,7 @@ sub extract_from_shlibs {
 	            $shlibfile, $., $_);
 	    next;
 	}
-	my $depread = defined($4) ? $4 : '';
+	my $depread = $4 // '';
 	if (($libname eq $2) && ($libversion eq $3)) {
 	    # Define dep and end here if the package type explicitly
 	    # matches. Otherwise if the packagetype is not specified, use
@@ -723,7 +725,7 @@ sub find_symbols_file {
     my ($pkg, $soname, $libfile) = @_;
     my @files;
     if ($pkg eq '') {
-	# If the file is not packaged, try to find out the symbols file in
+	# If the file is not packaged, try to find out the symbols file in
 	# the package being built where the lib has been found
 	my $pkg_root = guess_pkg_root_dir($libfile);
 	if (defined $pkg_root) {
@@ -753,8 +755,8 @@ sub symfile_has_soname {
         return $symfile_has_soname_cache{$file}{$soname};
     }
 
-    open(my $symfile_fh, '<', $file) ||
-        syserr(_g('cannot open file %s'), $file);
+    open(my $symfile_fh, '<', $file)
+        or syserr(_g('cannot open file %s'), $file);
     my $result = 0;
     while (<$symfile_fh>) {
 	if (/^\Q$soname\E /) {
@@ -845,19 +847,21 @@ sub find_packages {
     if (!$pid) {
 	# Child process running dpkg --search and discarding errors
 	close STDERR;
-	open STDERR, '>', '/dev/null';
+	open STDERR, '>', '/dev/null'
+	    or syserr(_g('cannot open file %s'), '/dev/null');
 	$ENV{LC_ALL} = 'C';
 	exec('dpkg', '--search', '--', @files)
-	    || syserr(_g('unable to execute %s'), 'dpkg');
+	    or syserr(_g('unable to execute %s'), 'dpkg');
     }
     while (defined($_ = <$dpkg_fh>)) {
 	chomp($_);
 	if (m/^local diversion |^diversion by/) {
 	    warning(_g('diversions involved - output may be incorrect'));
-	    print(STDERR " $_\n")
-		|| syserr(_g('write diversion info to stderr'));
+	    print { *STDERR } " $_\n"
+		or syserr(_g('write diversion info to stderr'));
 	} elsif (m/^([-a-z0-9+.:, ]+): (\/.*)$/) {
-	    $cached_pkgmatch{$2} = $pkgmatch->{$2} = [ split(/, /, $1) ];
+	    my ($pkgs, $path) = ($1, $2);
+	    $cached_pkgmatch{$path} = $pkgmatch->{$path} = [ split /, /, $pkgs ];
 	} else {
 	    warning(_g("unknown output from dpkg --search: '%s'"), $_);
 	}
