@@ -23,7 +23,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -42,7 +42,9 @@ use Dpkg::Substvars;
 use Dpkg::Version;
 use Dpkg::Vars;
 use Dpkg::Changelog::Parse;
-use Dpkg::Source::Package;
+use Dpkg::Source::Package qw(get_default_diff_ignore_regex
+                             set_default_diff_ignore_regex
+                             get_default_tar_ignore_pattern);
 use Dpkg::Vendor qw(run_vendor_hook);
 
 use Cwd;
@@ -57,13 +59,9 @@ my $changelogformat;
 
 my $build_format;
 my %options = (
-    # Compression related
-    compression => compression_get_default(),
-    comp_level => compression_get_default_level(),
-    comp_ext => compression_get_property(compression_get_default(), 'file_ext'),
     # Ignore files
     tar_ignore => [],
-    diff_ignore_regexp => '',
+    diff_ignore_regex => '',
     # Misc options
     copy_orig_tarballs => 1,
     no_check => 0,
@@ -76,6 +74,7 @@ my %override;
 
 my $substvars = Dpkg::Substvars->new();
 my $tar_ignore_default_pattern_done;
+my $diff_ignore_regex = get_default_diff_ignore_regex();
 
 my @options;
 my @cmdline_options;
@@ -85,7 +84,7 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
         setopmode('-b');
     } elsif (m/^-x$/) {
         setopmode('-x');
-    } elsif (m/^--(before|after)-build$/) {
+    } elsif (m/^--(?:before|after)-build$/) {
         setopmode($_);
     } elsif (m/^--commit$/) {
         setopmode($_);
@@ -107,14 +106,14 @@ if (defined($options{opmode}) &&
     } else {
 	$dir = File::Spec->catdir(shift(@ARGV));
     }
-    stat($dir) || syserr(_g('cannot stat directory %s'), $dir);
+    stat($dir) or syserr(_g('cannot stat directory %s'), $dir);
     if (not -d $dir) {
 	error(_g('directory argument %s is not a directory'), $dir);
     }
     if ($dir eq '.') {
 	# . is never correct, adjust automatically
 	$dir = basename(cwd());
-	chdir('..') || syserr(_g("unable to chdir to `%s'"), '..');
+	chdir('..') or syserr(_g("unable to chdir to `%s'"), '..');
     }
     # --format options are not allowed, they would take precedence
     # over real command line options, debian/source/format should be used
@@ -146,7 +145,6 @@ while (@options) {
     } elsif (m/^-(?:Z|-compression=)(.*)$/) {
 	my $compression = $1;
 	$options{compression} = $compression;
-	$options{comp_ext} = compression_get_property($compression, 'file_ext');
 	usageerr(_g('%s is not a supported compression'), $compression)
 	    unless compression_is_supported($compression);
 	compression_set_default($compression);
@@ -167,17 +165,18 @@ while (@options) {
     } elsif (m/^-U([^\=:]+)$/) {
         $remove{$1} = 1;
     } elsif (m/^-(?:i|-diff-ignore(?:$|=))(.*)$/) {
-        $options{diff_ignore_regexp} = $1 ? $1 : $Dpkg::Source::Package::diff_ignore_default_regexp;
+        $options{diff_ignore_regex} = $1 ? $1 : $diff_ignore_regex;
     } elsif (m/^--extend-diff-ignore=(.+)$/) {
-	$Dpkg::Source::Package::diff_ignore_default_regexp .= "|$1";
-	if ($options{diff_ignore_regexp}) {
-	    $options{diff_ignore_regexp} .= "|$1";
+	$diff_ignore_regex .= "|$1";
+	if ($options{diff_ignore_regex}) {
+	    $options{diff_ignore_regex} .= "|$1";
 	}
+	set_default_diff_ignore_regex($diff_ignore_regex);
     } elsif (m/^-(?:I|-tar-ignore=)(.+)$/) {
         push @{$options{tar_ignore}}, $1;
     } elsif (m/^-(?:I|-tar-ignore)$/) {
         unless ($tar_ignore_default_pattern_done) {
-            push @{$options{tar_ignore}}, @Dpkg::Source::Package::tar_ignore_default_pattern;
+            push @{$options{tar_ignore}}, get_default_tar_ignore_pattern();
             # Prevent adding multiple times
             $tar_ignore_default_pattern_done = 1;
         }
@@ -191,7 +190,7 @@ while (@options) {
         $substvars->set($1, $2);
     } elsif (m/^-T(.*)$/) {
 	$substvars->load($1) if -e $1;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-(?:\?|-help)$/) {
         usage();
         exit(0);
     } elsif (m/^--version$/) {
@@ -246,7 +245,7 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
 	    $fields->{$_} = $v;
 	} elsif (m/^Uploaders$/i) {
 	    ($fields->{$_} = $v) =~ s/\s*[\r\n]\s*/ /g; # Merge in a single-line
-	} elsif (m/^Build-(Depends|Conflicts)(-Arch|-Indep)?$/i) {
+	} elsif (m/^Build-(?:Depends|Conflicts)(?:-Arch|-Indep)?$/i) {
 	    my $dep;
 	    my $type = field_get_dep_type($_);
 	    $dep = deps_parse($v, build_dep => 1, union => $type eq 'union');
@@ -268,8 +267,17 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
 	my $prio = $pkg->{'Priority'} || $src_prio;
 	my $type = $pkg->{'Package-Type'} ||
 	        $pkg->get_custom_field('Package-Type') || 'deb';
-	push @pkglist, sprintf('%s %s %s %s', $p, $type, $sect, $prio);
-	push(@binarypackages,$p);
+        my $arch = $pkg->{'Architecture'};
+        my $profile = $pkg->{'Build-Profiles'};
+
+        my $pkg_summary = sprintf('%s %s %s %s', $p, $type, $sect, $prio);
+
+        $pkg_summary .= ' arch=' . join ',', split /\s+/, $arch;
+        $pkg_summary .= ' profile=' . join ',', split /\s+/, $profile
+            if defined $profile;
+
+        push @pkglist, $pkg_summary;
+	push @binarypackages, $p;
 	foreach (keys %{$pkg}) {
 	    my $v = $pkg->{$_};
             if (m/^Architecture$/) {
@@ -352,8 +360,8 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
     # Select the format to use
     if (not defined $build_format) {
 	if (-e "$dir/debian/source/format") {
-	    open(my $format_fh, '<', "$dir/debian/source/format") ||
-		syserr(_g('cannot read %s'), "$dir/debian/source/format");
+	    open(my $format_fh, '<', "$dir/debian/source/format")
+	        or syserr(_g('cannot read %s'), "$dir/debian/source/format");
 	    $build_format = <$format_fh>;
 	    chomp($build_format) if defined $build_format;
 	    error(_g('%s is empty'), "$dir/debian/source/format")
@@ -398,7 +406,7 @@ if ($options{opmode} =~ /^(-b|--print-format|--(before|after)-build|--commit)$/)
 
     # Write the .dsc
     my $dscname = $srcpkg->get_basename(1) . '.dsc';
-    info(_g('building %s in %s'), $sourcepackage, $dscname);
+    info(_g('building %s in %s'), get_source_package(), $dscname);
     $srcpkg->write_dsc(filename => $dscname,
 		       remove => \%remove,
 		       override => \%override,
@@ -495,7 +503,7 @@ sub usage {
   -D<field>=<value>        override or add a .dsc field and value.
   -U<field>                remove a field.
   -q                       quiet mode.
-  -i[<regexp>]             filter out files to ignore diffs of
+  -i[<regex>]              filter out files to ignore diffs of
                              (defaults to: '%s').
   -I[<pattern>]            filter out files when building tarballs
                              (defaults to: %s).
@@ -516,8 +524,8 @@ sub usage {
 'More options are available but they depend on the source package format.
 See dpkg-source(1) for more info.') . "\n",
     $Dpkg::PROGNAME,
-    $Dpkg::Source::Package::diff_ignore_default_regexp,
-    join(' ', map { "-I$_" } @Dpkg::Source::Package::tar_ignore_default_pattern),
+    get_default_diff_ignore_regex(),
+    join(' ', map { "-I$_" } get_default_tar_ignore_pattern()),
     compression_get_default(),
     join(' ', compression_get_list()),
     compression_get_default_level();
