@@ -17,7 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -42,7 +42,6 @@
 #include "filesdb.h"
 #include "main.h"
 
-static FILE *statoverridefile = NULL;
 static char *statoverridename;
 
 uid_t
@@ -111,15 +110,17 @@ statdb_parse_mode(const char *str)
 void
 ensure_statoverrides(void)
 {
-	struct stat stab1, stab2;
+	static struct stat sb_prev;
+	struct stat sb_next;
+	static FILE *file_prev;
 	FILE *file;
 	char *loaded_list, *loaded_list_end, *thisline, *nextline, *ptr;
 	struct file_stat *fso;
 	struct filenamenode *fnn;
+	struct fileiterator *iter;
 
-	if (statoverridename != NULL)
-		free(statoverridename);
-	statoverridename = dpkg_db_get_path(STATOVERRIDEFILE);
+	if (statoverridename == NULL)
+		statoverridename = dpkg_db_get_path(STATOVERRIDEFILE);
 
 	onerr_abort++;
 
@@ -127,40 +128,57 @@ ensure_statoverrides(void)
 	if (!file) {
 		if (errno != ENOENT)
 			ohshite(_("failed to open statoverride file"));
-		if (!statoverridefile) {
+	} else {
+		setcloexec(fileno(file), statoverridename);
+
+		if (fstat(fileno(file), &sb_next))
+			ohshite(_("failed to fstat statoverride file"));
+
+		/*
+		 * We need to keep the database file open so that the
+		 * filesystem cannot reuse the inode number (f.ex. during
+		 * multiple dpkg-statoverride invocations in a maintainer
+		 * script), otherwise the following check might turn true,
+		 * and we would skip reloading a modified database.
+		 */
+		if (file_prev &&
+		    sb_prev.st_dev == sb_next.st_dev &&
+		    sb_prev.st_ino == sb_next.st_ino) {
+			fclose(file);
 			onerr_abort--;
+			debug(dbg_general, "%s: same, skipping", __func__);
 			return;
 		}
-	} else {
-		if (fstat(fileno(file), &stab2))
-			ohshite(_("failed to fstat statoverride file"));
-		if (statoverridefile) {
-			if (fstat(fileno(statoverridefile), &stab1))
-				ohshite(_("failed to fstat previous statoverride file"));
-			if (stab1.st_dev == stab2.st_dev &&
-			    stab1.st_ino == stab2.st_ino) {
-				fclose(file);
-				onerr_abort--;
-				return;
-			}
-		}
+		sb_prev = sb_next;
 	}
-	if (statoverridefile)
-		fclose(statoverridefile);
-	statoverridefile = file;
-	setcloexec(fileno(statoverridefile), statoverridename);
+	if (file_prev)
+		fclose(file_prev);
+	file_prev = file;
+
+	/* Reset statoverride information. */
+	iter = files_db_iter_new();
+	while ((fnn = files_db_iter_next(iter)))
+		fnn->statoverride = NULL;
+	files_db_iter_free(iter);
+
+	if (!file) {
+		onerr_abort--;
+		debug(dbg_general, "%s: none, resetting", __func__);
+		return;
+	}
+	debug(dbg_general, "%s: new, (re)loading", __func__);
 
 	/* If the statoverride list is empty we don't need to bother
 	 * reading it. */
-	if (!stab2.st_size) {
+	if (!sb_next.st_size) {
 		onerr_abort--;
 		return;
 	}
 
-	loaded_list = nfmalloc(stab2.st_size);
-	loaded_list_end = loaded_list + stab2.st_size;
+	loaded_list = nfmalloc(sb_next.st_size);
+	loaded_list_end = loaded_list + sb_next.st_size;
 
-	if (fd_read(fileno(file), loaded_list, stab2.st_size) < 0)
+	if (fd_read(fileno(file), loaded_list, sb_next.st_size) < 0)
 		ohshite(_("reading statoverride file '%.250s'"), statoverridename);
 
 	thisline = loaded_list;
