@@ -39,43 +39,34 @@
 #  define OSFreeBSD
 #elif defined(__NetBSD__)
 #  define OSNetBSD
+#elif defined(__DragonFly__)
+#  define OSDragonFlyBSD
 #else
 #  error Unknown architecture - cannot build start-stop-daemon
 #endif
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 #ifdef HAVE_SYS_SYSCALL_H
 #include <sys/syscall.h>
 #endif
-
-#if defined(OSHurd)
-#include <hurd.h>
-#include <ps.h>
-#endif
-
-#if defined(OSOpenBSD) || defined(OSFreeBSD) || defined(OSNetBSD)
-#include <sys/param.h>
-#include <sys/proc.h>
-
-#include <err.h>
-#endif
-
-#ifdef HAVE_KVM_H
+#ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
-#include <sys/user.h>
-
-#include <kvm.h>
 #endif
-
-#if defined(OShpux)
-#include <sys/param.h>
+#ifdef HAVE_SYS_PROC_H
+#include <sys/proc.h>
+#endif
+#ifdef HAVE_SYS_USER_H
+#include <sys/user.h>
+#endif
+#ifdef HAVE_SYS_PSTAT_H
 #include <sys/pstat.h>
 #endif
-
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/termios.h>
 
 #include <assert.h>
 #include <errno.h>
@@ -87,6 +78,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include <signal.h>
+#include <termios.h>
 #include <unistd.h>
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
@@ -96,10 +88,21 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <getopt.h>
-
 #ifdef HAVE_ERROR_H
 #include <error.h>
 #endif
+#ifdef HAVE_ERR_H
+#include <err.h>
+#endif
+
+#if defined(OSHurd)
+#include <hurd.h>
+#include <ps.h>
+#endif
+
+#ifdef HAVE_KVM_H
+#include <kvm.h>
+#endif
 
 #ifdef _POSIX_PRIORITY_SCHEDULING
 #include <sched.h>
@@ -122,6 +125,9 @@
 #define PROCESS_NAME_SIZE 16
 #elif defined(OSFreeBSD)
 #define PROCESS_NAME_SIZE 19
+#elif defined(OSDragonFlyBSD)
+/* On DragonFlyBSD MAXCOMLEN expands to 16. */
+#define PROCESS_NAME_SIZE MAXCOMLEN
 #endif
 
 #define MIN_POLL_INTERVAL 20000 /* µs */
@@ -149,19 +155,19 @@ enum {
 };
 
 enum action_code {
-	action_none,
-	action_start,
-	action_stop,
-	action_status,
+	ACTION_NONE,
+	ACTION_START,
+	ACTION_STOP,
+	ACTION_STATUS,
 };
 
 static enum action_code action;
-static int testmode = 0;
+static bool testmode = false;
 static int quietmode = 0;
 static int exitnodo = 1;
-static int background = 0;
-static int close_io = 1;
-static int mpidfile = 0;
+static bool background = false;
+static bool close_io = true;
+static bool mpidfile = false;
 static int signal_nr = SIGTERM;
 static int user_id = -1;
 static int runas_uid = -1;
@@ -174,6 +180,8 @@ static const char *changedir = "/";
 static const char *cmdname = NULL;
 static char *execname = NULL;
 static char *startas = NULL;
+static pid_t match_pid = -1;
+static pid_t match_ppid = -1;
 static const char *pidfile = NULL;
 static char what_stop[1024];
 static const char *progname = "";
@@ -187,11 +195,11 @@ static struct proc_stat_list *procset = NULL;
 
 /* LSB Init Script process status exit codes. */
 enum status_code {
-	status_ok = 0,
-	status_dead_pidfile = 1,
-	status_dead_lockfile = 2,
-	status_dead = 3,
-	status_unknown = 4,
+	STATUS_OK = 0,
+	STATUS_DEAD_PIDFILE = 1,
+	STATUS_DEAD_LOCKFILE = 2,
+	STATUS_DEAD = 3,
+	STATUS_UNKNOWN = 4,
 };
 
 struct pid_list {
@@ -254,8 +262,8 @@ fatal(const char *format, ...)
 	else
 		fprintf(stderr, "\n");
 
-	if (action == action_status)
-		exit(status_unknown);
+	if (action == ACTION_STATUS)
+		exit(STATUS_UNKNOWN);
 	else
 		exit(2);
 }
@@ -437,6 +445,8 @@ usage(void)
 "  -V|--version                  print version\n"
 "\n"
 "Matching options (at least one is required):\n"
+"     --pid <pid>                pid to check\n"
+"     --ppid <ppid>              parent pid to check\n"
 "  -p|--pidfile <pid-file>       pid file to check\n"
 "  -x|--exec <executable>        program to start/check if it is running\n"
 "  -n|--name <process-name>      process name to check\n"
@@ -506,8 +516,8 @@ badusage(const char *msg)
 		fprintf(stderr, "%s: %s\n", progname, msg);
 	fprintf(stderr, "Try '%s --help' for more information.\n", progname);
 
-	if (action == action_status)
-		exit(status_unknown);
+	if (action == ACTION_STATUS)
+		exit(STATUS_UNKNOWN);
 	else
 		exit(3);
 }
@@ -559,6 +569,17 @@ parse_unsigned(const char *string, int base, int *value_r)
 	return 0;
 }
 
+static int
+parse_pid(const char *pid_str, int *pid_num)
+{
+	if (parse_unsigned(pid_str, 10, pid_num) != 0)
+		return -1;
+	if (*pid_num == 0)
+		return -1;
+
+	return 0;
+}
+
 static int
 parse_signal(const char *sig_str, int *sig_num)
 {
@@ -738,8 +759,8 @@ parse_schedule(const char *schedule_str)
 		schedule[0].value = signal_nr;
 		parse_schedule_item(schedule_str, &schedule[1]);
 		if (schedule[1].type != sched_timeout) {
-			badusage ("--retry takes timeout, or schedule list"
-			          " of at least two items");
+			badusage("--retry takes timeout, or schedule list"
+			         " of at least two items");
 		}
 		schedule[2].type = sched_signal;
 		schedule[2].value = SIGKILL;
@@ -785,12 +806,15 @@ set_action(enum action_code new_action)
 	if (action == new_action)
 		return;
 
-	if (action != action_none)
+	if (action != ACTION_NONE)
 		badusage("only one command can be specified");
 
 	action = new_action;
 }
 
+#define OPT_PID		500
+#define OPT_PPID	501
+
 static void
 parse_options(int argc, char * const *argv)
 {
@@ -803,6 +827,8 @@ parse_options(int argc, char * const *argv)
 		{ "startas",	  1, NULL, 'a'},
 		{ "name",	  1, NULL, 'n'},
 		{ "oknodo",	  0, NULL, 'o'},
+		{ "pid",	  1, NULL, OPT_PID},
+		{ "ppid",	  1, NULL, OPT_PPID},
 		{ "pidfile",	  1, NULL, 'p'},
 		{ "quiet",	  0, NULL, 'q'},
 		{ "signal",	  1, NULL, 's'},
@@ -824,6 +850,8 @@ parse_options(int argc, char * const *argv)
 		{ "chdir",	  1, NULL, 'd'},
 		{ NULL,		  0, NULL, 0  }
 	};
+	const char *pid_str = NULL;
+	const char *ppid_str = NULL;
 	const char *umask_str = NULL;
 	const char *signal_str = NULL;
 	const char *schedule_str = NULL;
@@ -833,7 +861,7 @@ parse_options(int argc, char * const *argv)
 
 	for (;;) {
 		c = getopt_long(argc, argv,
-		                "HKSVTa:n:op:qr:s:tu:vx:c:N:P:I:k:bmR:g:d:",
+		                "HKSVTa:n:op:qr:s:tu:vx:c:N:P:I:k:bCmR:g:d:",
 		                longopts, NULL);
 		if (c == -1)
 			break;
@@ -842,13 +870,13 @@ parse_options(int argc, char * const *argv)
 			usage();
 			exit(0);
 		case 'K':  /* --stop */
-			set_action(action_stop);
+			set_action(ACTION_STOP);
 			break;
 		case 'S':  /* --start */
-			set_action(action_start);
+			set_action(ACTION_START);
 			break;
 		case 'T':  /* --status */
-			set_action(action_status);
+			set_action(ACTION_STATUS);
 			break;
 		case 'V':  /* --version */
 			do_version();
@@ -862,17 +890,23 @@ parse_options(int argc, char * const *argv)
 		case 'o':  /* --oknodo */
 			exitnodo = 0;
 			break;
+		case OPT_PID: /* --pid <pid> */
+			pid_str = optarg;
+			break;
+		case OPT_PPID: /* --ppid <ppid> */
+			ppid_str = optarg;
+			break;
 		case 'p':  /* --pidfile <pid-file> */
 			pidfile = optarg;
 			break;
 		case 'q':  /* --quiet */
-			quietmode = 1;
+			quietmode = true;
 			break;
 		case 's':  /* --signal <signal> */
 			signal_str = optarg;
 			break;
 		case 't':  /* --test */
-			testmode = 1;
+			testmode = true;
 			break;
 		case 'u':  /* --user <username>|<uid> */
 			userspec = optarg;
@@ -909,13 +943,13 @@ parse_options(int argc, char * const *argv)
 			umask_str = optarg;
 			break;
 		case 'b':  /* --background */
-			background = 1;
+			background = true;
 			break;
 		case 'C': /* --no-close */
-			close_io = 0;
+			close_io = false;
 			break;
 		case 'm':  /* --make-pidfile */
-			mpidfile = 1;
+			mpidfile = true;
 			break;
 		case 'R':  /* --retry <schedule>|<timeout> */
 			schedule_str = optarg;
@@ -929,6 +963,16 @@ parse_options(int argc, char * const *argv)
 		}
 	}
 
+	if (pid_str != NULL) {
+		if (parse_pid(pid_str, &match_pid) != 0)
+			badusage("pid value must be a number greater than 0");
+	}
+
+	if (ppid_str != NULL) {
+		if (parse_pid(ppid_str, &match_ppid) != 0)
+			badusage("ppid value must be a number greater than 0");
+	}
+
 	if (signal_str != NULL) {
 		if (parse_signal(signal_str, &signal_nr) != 0)
 			badusage("signal value must be numeric or name"
@@ -950,11 +994,12 @@ parse_options(int argc, char * const *argv)
 			badusage("umask value must be a positive number");
 	}
 
-	if (action == action_none)
+	if (action == ACTION_NONE)
 		badusage("need one of --start or --stop or --status");
 
-	if (!execname && !pidfile && !userspec && !cmdname)
-		badusage("need at least one of --exec, --pidfile, --user or --name");
+	if (!execname && !pid_str && !ppid_str && !pidfile && !userspec &&
+	    !cmdname)
+		badusage("need at least one of --exec, --pid, --ppid, --pidfile, --user or --name");
 
 #ifdef PROCESS_NAME_SIZE
 	if (cmdname && strlen(cmdname) > PROCESS_NAME_SIZE)
@@ -966,13 +1011,16 @@ parse_options(int argc, char * const *argv)
 	if (!startas)
 		startas = execname;
 
-	if (action == action_start && !startas)
+	if (action == ACTION_START && !startas)
 		badusage("--start needs --exec or --startas");
 
 	if (mpidfile && pidfile == NULL)
 		badusage("--make-pidfile requires --pidfile");
 
-	if (background && action != action_start)
+	if (pid_str && pidfile)
+		badusage("need either --pid of --pidfile, not both");
+
+	if (background && action != ACTION_START)
 		badusage("--background is only relevant with --start");
 
 	if (!close_io && !background)
@@ -1042,6 +1090,40 @@ setup_options(void)
 	}
 }
 
+#if defined(OSLinux)
+static const char *
+proc_status_field(pid_t pid, const char *field)
+{
+	static char *line = NULL;
+	static size_t line_size = 0;
+
+	FILE *fp;
+	char filename[32];
+	char *value = NULL;
+	ssize_t line_len;
+	size_t field_len = strlen(field);
+
+	sprintf(filename, "/proc/%d/status", pid);
+	fp = fopen(filename, "r");
+	if (!fp)
+		return NULL;
+	while ((line_len = getline(&line, &line_size, fp)) >= 0) {
+		if (strncasecmp(line, field, field_len) == 0) {
+			line[line_len - 1] = '\0';
+
+			value = line + field_len;
+			while (isspace(*value))
+				value++;
+
+			break;
+		}
+	}
+	fclose(fp);
+
+	return value;
+}
+#endif
+
 #if defined(OSHurd)
 static void
 init_procset(void)
@@ -1088,21 +1170,29 @@ static bool
 pid_is_exec(pid_t pid, const struct stat *esb)
 {
 	char lname[32];
-	char lcontents[_POSIX_PATH_MAX];
+	char lcontents[_POSIX_PATH_MAX + 1];
+	char *filename;
 	const char deleted[] = " (deleted)";
 	int nread;
 	struct stat sb;
 
 	sprintf(lname, "/proc/%d/exe", pid);
-	nread = readlink(lname, lcontents, sizeof(lcontents));
+	nread = readlink(lname, lcontents, sizeof(lcontents) - 1);
 	if (nread == -1)
 		return false;
 
-	lcontents[nread] = '\0';
-	if (strcmp(lcontents + nread - strlen(deleted), deleted) == 0)
-		lcontents[nread - strlen(deleted)] = '\0';
+	filename = lcontents;
+	filename[nread] = '\0';
 
-	if (stat(lcontents, &sb) != 0)
+	/* OpenVZ kernels contain a bogus patch that instead of appending,
+	 * prepends the deleted marker. Workaround those. Otherwise handle
+	 * the normal appended marker. */
+	if (strncmp(filename, deleted, strlen(deleted)) == 0)
+		filename += strlen(deleted);
+	else if (strcmp(filename + nread - strlen(deleted), deleted) == 0)
+		filename[nread - strlen(deleted)] = '\0';
+
+	if (stat(filename, &sb) != 0)
 		return false;
 
 	return (sb.st_dev == esb->st_dev && sb.st_ino == esb->st_ino);
@@ -1137,6 +1227,32 @@ pid_is_exec(pid_t pid, const struct stat *esb)
 	return ((dev_t)pst.pst_text.psf_fsid.psfs_id == esb->st_dev &&
 	        (ino_t)pst.pst_text.psf_fileid == esb->st_ino);
 }
+#elif defined(OSFreeBSD)
+static bool
+pid_is_exec(pid_t pid, const struct stat *esb)
+{
+	struct stat sb;
+	int error, name[4];
+	size_t len;
+	char pathname[PATH_MAX];
+
+	name[0] = CTL_KERN;
+	name[1] = KERN_PROC;
+	name[2] = KERN_PROC_PATHNAME;
+	name[3] = pid;
+	len = sizeof(pathname);
+
+	error = sysctl(name, 4, pathname, &len, NULL, 0);
+	if (error != 0 && errno != ESRCH)
+		return false;
+	if (len == 0)
+		pathname[0] = '\0';
+
+	if (stat(pathname, &sb) != 0)
+		return false;
+
+	return (sb.st_dev == esb->st_dev && sb.st_ino == esb->st_ino);
+}
 #elif defined(HAVE_KVM_H)
 static bool
 pid_is_exec(pid_t pid, const struct stat *esb)
@@ -1184,6 +1300,81 @@ pid_is_exec(pid_t pid, const struct stat *esb)
 }
 #endif
 
+#if defined(OSLinux)
+static bool
+pid_is_child(pid_t pid, pid_t ppid)
+{
+	const char *ppid_str;
+	pid_t proc_ppid;
+	int rc;
+
+	ppid_str = proc_status_field(pid, "PPid:");
+	if (ppid_str == NULL)
+		return false;
+
+	rc = parse_pid(ppid_str, &proc_ppid);
+	if (rc < 0)
+		return false;
+
+	return proc_ppid == ppid;
+}
+#elif defined(OSHurd)
+static bool
+pid_is_child(pid_t pid, pid_t ppid)
+{
+	struct proc_stat *ps;
+	struct procinfo *pi;
+
+	ps = get_proc_stat(pid, PSTAT_PROC_INFO);
+	if (ps == NULL)
+		return false;
+
+	pi = proc_stat_proc_info(ps);
+
+	return pi->ppid == ppid;
+}
+#elif defined(OShpux)
+static bool
+pid_is_child(pid_t pid, pid_t ppid)
+{
+	struct pst_status pst;
+
+	if (pstat_getproc(&pst, sizeof(pst), (size_t)0, (int)pid) < 0)
+		return false;
+
+	return pst.pst_ppid == ppid;
+}
+#elif defined(HAVE_KVM_H)
+static bool
+pid_is_child(pid_t pid, pid_t ppid)
+{
+	kvm_t *kd;
+	int nentries;
+	struct kinfo_proc *kp;
+	char errbuf[_POSIX2_LINE_MAX];
+	pid_t proc_ppid;
+
+	kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf);
+	if (kd == NULL)
+		errx(1, "%s", errbuf);
+	kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &nentries);
+	if (kp == NULL)
+		errx(1, "%s", kvm_geterr(kd));
+
+#if defined(OSFreeBSD)
+	proc_ppid = kp->ki_ppid;
+#elif defined(OSOpenBSD)
+	proc_ppid = kp->p_ppid;
+#elif defined(OSDragonFlyBSD)
+	proc_ppid = kp->kp_ppid;
+#else
+	proc_ppid = kp->kp_proc.p_ppid;
+#endif
+
+	return proc_ppid == ppid;
+}
+#endif
+
 #if defined(OSLinux)
 static bool
 pid_is_user(pid_t pid, uid_t uid)
@@ -1231,11 +1422,21 @@ pid_is_user(pid_t pid, uid_t uid)
 	kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &nentries);
 	if (kp == NULL)
 		errx(1, "%s", kvm_geterr(kd));
+
+#if defined(OSFreeBSD)
+	proc_uid = kp->ki_ruid;
+#elif defined(OSOpenBSD)
+	proc_uid = kp->p_ruid;
+#elif defined(OSDragonFlyBSD)
+	proc_uid = kp->kp_ruid;
+#else
 	if (kp->kp_proc.p_cred)
 		kvm_read(kd, (u_long)&(kp->kp_proc.p_cred->p_ruid),
 		         &proc_uid, sizeof(uid_t));
 	else
 		return false;
+#endif
+
 	return (proc_uid == (uid_t)uid);
 }
 #endif
@@ -1244,25 +1445,13 @@ pid_is_user(pid_t pid, uid_t uid)
 static bool
 pid_is_cmd(pid_t pid, const char *name)
 {
-	char buf[32];
-	FILE *f;
-	int c;
+	const char *comm;
 
-	sprintf(buf, "/proc/%d/stat", pid);
-	f = fopen(buf, "r");
-	if (!f)
+	comm = proc_status_field(pid, "Name:");
+	if (comm == NULL)
 		return false;
-	while ((c = getc(f)) != EOF && c != '(')
-		;
-	if (c != '(') {
-		fclose(f);
-		return false;
-	}
-	/* This hopefully handles command names containing ‘)’. */
-	while ((c = getc(f)) != EOF && c == *name)
-		name++;
-	fclose(f);
-	return (c == ')' && *name == '\0');
+
+	return strcmp(comm, name) == 0;
 }
 #elif defined(OSHurd)
 static bool
@@ -1320,7 +1509,17 @@ pid_is_cmd(pid_t pid, const char *name)
 	kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &nentries);
 	if (kp == NULL)
 		errx(1, "%s", kvm_geterr(kd));
-	process_name = (&kp->kp_proc)->p_comm;
+
+#if defined(OSFreeBSD)
+	process_name = kp->ki_comm;
+#elif defined(OSOpenBSD)
+	process_name = kp->p_comm;
+#elif defined(OSDragonFlyBSD)
+	process_name = kp->kp_comm;
+#else
+	process_name = kp->kp_proc.p_comm;
+#endif
+
 	if (strlen(name) != strlen(process_name))
 		return false;
 	return (strcmp(name, process_name) == 0);
@@ -1350,17 +1549,19 @@ static enum status_code
 pid_check(pid_t pid)
 {
 	if (execname && !pid_is_exec(pid, &exec_stat))
-		return status_dead;
+		return STATUS_DEAD;
+	if (match_ppid > 0 && !pid_is_child(pid, match_ppid))
+		return STATUS_DEAD;
 	if (userspec && !pid_is_user(pid, user_id))
-		return status_dead;
+		return STATUS_DEAD;
 	if (cmdname && !pid_is_cmd(pid, cmdname))
-		return status_dead;
-	if (action != action_stop && !pid_is_running(pid))
-		return status_dead;
+		return STATUS_DEAD;
+	if (action != ACTION_STOP && !pid_is_running(pid))
+		return STATUS_DEAD;
 
 	pid_list_push(&found, pid);
 
-	return status_ok;
+	return STATUS_OK;
 }
 
 static enum status_code
@@ -1379,15 +1580,15 @@ do_pidfile(const char *name)
 		if (fscanf(f, "%d", &pid) == 1)
 			pid_status = pid_check(pid);
 		else
-			pid_status = status_unknown;
+			pid_status = STATUS_UNKNOWN;
 		fclose(f);
 
-		if (pid_status == status_dead)
-			return status_dead_pidfile;
+		if (pid_status == STATUS_DEAD)
+			return STATUS_DEAD_PIDFILE;
 		else
 			return pid_status;
 	} else if (errno == ENOENT)
-		return status_dead;
+		return STATUS_DEAD;
 	else
 		fatal("unable to open pidfile %s", name);
 }
@@ -1400,7 +1601,7 @@ do_procinit(void)
 	struct dirent *entry;
 	int foundany;
 	pid_t pid;
-	enum status_code prog_status = status_dead;
+	enum status_code prog_status = STATUS_DEAD;
 
 	procdir = opendir("/proc");
 	if (!procdir)
@@ -1441,9 +1642,9 @@ do_procinit(void)
 	proc_stat_list_for_each(procset, check_proc_stat);
 
 	if (found)
-		return status_ok;
+		return STATUS_OK;
 	else
-		return status_dead;
+		return STATUS_DEAD;
 }
 #elif defined(OShpux)
 static enum status_code
@@ -1452,7 +1653,7 @@ do_procinit(void)
 	struct pst_status pst[10];
 	int i, count;
 	int idx = 0;
-	enum status_code prog_status = status_dead;
+	enum status_code prog_status = STATUS_DEAD;
 
 	while ((count = pstat_getproc(pst, sizeof(pst[0]), 10, idx)) > 0) {
 		enum status_code pid_status;
@@ -1471,8 +1672,41 @@ do_procinit(void)
 static enum status_code
 do_procinit(void)
 {
-	/* Nothing to do. */
-	return status_unknown;
+	kvm_t *kd;
+	int nentries, i;
+	struct kinfo_proc *kp;
+	char errbuf[_POSIX2_LINE_MAX];
+	enum status_code prog_status = STATUS_DEAD;
+
+	kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf);
+	if (kd == NULL)
+		errx(1, "%s", errbuf);
+	kp = kvm_getprocs(kd, KERN_PROC_ALL, 0, &nentries);
+	if (kp == NULL)
+		errx(1, "%s", kvm_geterr(kd));
+
+	for (i = 0; i < nentries; i++) {
+		enum status_code pid_status;
+		pid_t pid;
+
+#if defined(OSFreeBSD)
+		pid = kp[i].ki_pid;
+#elif defined(OSOpenBSD)
+		pid = kp[i].p_pid;
+#elif defined(OSDragonFlyBSD)
+		pid = kp[i].kp_pid;
+#else
+		pid = kp[i].kp_proc.p_pid;
+#endif
+
+		pid_status = pid_check(pid);
+		if (pid_status < prog_status)
+			prog_status = pid_status;
+	}
+
+	kvm_close(kd);
+
+	return prog_status;
 }
 #endif
 
@@ -1481,7 +1715,9 @@ do_findprocs(void)
 {
 	pid_list_free(&found);
 
-	if (pidfile)
+	if (match_pid > 0)
+		return pid_check(match_pid);
+	else if (pidfile)
 		return do_pidfile(pidfile);
 	else
 		return do_procinit();
@@ -1681,7 +1917,7 @@ static bool
 do_stop_timeout(int timeout, int *n_killed, int *n_notkilled)
 {
 	struct timeval stopat, before, after, interval, maxinterval;
-	int r, ratio;
+	int rc, ratio;
 
 	xgettimeofday(&stopat);
 	stopat.tv_sec += timeout;
@@ -1717,8 +1953,8 @@ do_stop_timeout(int timeout, int *n_killed, int *n_notkilled)
 		    interval.tv_usec <= MIN_POLL_INTERVAL)
 			interval.tv_usec = MIN_POLL_INTERVAL;
 
-		r = select(0, NULL, NULL, NULL, &interval);
-		if (r < 0 && errno != EINTR)
+		rc = select(0, NULL, NULL, NULL, &interval);
+		if (rc < 0 && errno != EINTR)
 			fatal("select() failed for pause");
 	}
 }
@@ -1818,15 +2054,15 @@ main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if (action == action_start)
+	if (action == ACTION_START)
 		do_start(argc, argv);
 
-	if (action == action_stop) {
+	if (action == ACTION_STOP) {
 		int i = run_stop_schedule();
 		exit(i);
 	}
 
-	if (action == action_status) {
+	if (action == ACTION_STATUS) {
 		enum status_code prog_status;
 
 		prog_status = do_findprocs();
